set mprint=no.

*** Created by Eveline Gebhardt, Australian Council for Educational Research ***.

define MIXED_noPV (infile = !charend('/')/
                   fixef =  !default(EMPTY) !charend('/')/
                   ranef = !default(NORAN) !charend('/')/
                   intef = !default(NOINT) !charend('/')/
                   dep = !charend('/')/
                   level2 = !default(SCHOOLID) !charend('/')/
                   grp = !default(NOGRP) !charend('/')/
                   nrep = !default(80) !charend('/')/
                   wgt = !default(W_FSTUWT) !charend('/')/
                   rwgt = !default(W_FSTR) !charend('/')/
                   cons = !default(0.05) !charend('/')).

preserve.
SET Olang=English OVars names ONumbers values TVars names TNumbers values.
omsend.
oms /select headings tables texts/destination viewer=no /tag="crap".

!let !fixup=!upcase(!fixef).
!let !ranup=!upcase(!ranef).
!let !intup=!upcase(!intef).
!let !grpup=!upcase(!grp).

* nGrp is the number of group variables.
!let !nGrp=!null
!if (!upcase(!grp)=NOGRP) !then !else
!do !g !in (!grp)
!let !nGrp=!concat(!nGrp," ")
!doend !ifend.
!let !nGrpp=!length(!concat(!ngrp,!blanks(1))).
!let !nGrp=!length(!nGrp).


*** PREPARE DATA ***.

DATASET CLOSE ALL.
get file=!quote(!infile) /keep=!level2 !if (!fixup=EMPTY) !then !else !fixef !ifend !dep !if (!grpup=NOGRP) !then !else !grp !ifend
         !wgt !concat(!rwgt,'1') to !concat(!rwgt,!nrep).
!if (!grpup=NOGRP) !then compute NOGRP=1 !ifend.

* Normalise weights.
weight by !wgt.
aggregate /break=!grp /POPsize=N /SAMPsize=NU.
compute  !wgt=(!wgt/POPsize)*SAMPsize.
do repeat a= !concat(!rwgt,'1') to !concat(!rwgt,!nrep).
compute a=(a/POPsize)*SAMPsize.
end repeat.
weight off.

* Listwise deletion.
compute DEL=0.
!if (!fixup=EMPTY) !then !else
!do !f !in (!fixef).
if (missing(!f)) DEL=1.
!doend !ifend.
if (missing(!dep)) DEL=1.
select if DEL=0.
sort cases by !grp !level2.
save outfile='C:\temp\norm.sav' /drop=DEL SAMPsize POPsize.

DATASET CLOSE ALL.
get file='C:\temp\norm.sav'.


*** COMPUTE ESTIMATES ***.

split file layered by !grp.
oms /select tables /if subtype='Parameter Estimates' /destination format=sav outfile='c:\temp\fixful.sav' /tag='ff'.
oms /select tables /if subtype='Covariance Parameter Estimates' /destination format=sav outfile='c:\temp\ranful.sav' /tag='rf'.
weight off.
MIXED !dep !if (!fixup=EMPTY) !then !else with !fixef !ifend
  /FIXED=INTERCEPT !if (!fixup=EMPTY) !then !else !fixef !ifend !if (!intup=NOINT) !then !else !intef !ifend
  /METHOD=ML
  /PRINT=G SOLUTION 
  /RANDOM=INTERCEPT !if (!ranup=NORAN) !then !else !ranef !ifend | SUBJECT(!level2)
  /REGWGT=!wgt.
omsend tag='ff'.
omsend tag='rf'.
split file off.

DATASET CLOSE ALL.
get file='c:\temp\fixful.sav'
 /keep=label_ !if (!grpup=NOGRP) !then Var2 !else Var1 to !concat('var',!nGrpp) !ifend Estimate.
rename var (label_ !if (!grpup=NOGRP) !then Var2 !else Var1 to !concat('var',!nGrpp) !ifend Estimate
 =EFFECT !if (!grpup=NOGRP) !then PARAMETER !else !GRPUP PARAMETER !ifend STAT).
alter type EFFECT PARAMETER (a50).
SAVE OUTFILE='C:\TEMP\1.SAV'.

DATASET CLOSE ALL.
get file='c:\temp\ranful.sav'
 /keep=label_ !if (!grpup=NOGRP) !then Var2 !else Var1 to !concat('var',!nGrpp) !ifend Estimate.
rename var (label_ !if (!grpup=NOGRP) !then Var2 !else Var1 to !concat('var',!nGrpp) !ifend Estimate
 =EFFECT !if (!grpup=NOGRP) !then PARAMETER !else !GRPUP PARAMETER !ifend STAT).
alter type EFFECT PARAMETER (a50).
SAVE OUTFILE='C:\TEMP\2.SAV'.

aggregate /break=effect !if (!ngrp=0) !then !else !grp !ifend /TOTAL=sum(STAT).
select if substr(PARAMETER,1,9)='Intercept'.
compute STAT=STAT/TOTAL.
compute PARAMETER = '  Intra-class correlation'.
exe.
delete var total.

ADD FILES FILE='C:\TEMP\1.SAV' /FILE='C:\TEMP\2.SAV' /FILE=*.
sort cases by !if (!ngrp=0) !then !else !grp !ifend EFFECT PARAMETER.
SAVE OUTFILE=!quote(!concat('C:\TEMP\',!wgt,'.sav')).


*** REPLICATES ***.

DATASET CLOSE ALL.
get file='C:\temp\norm.sav'.
split file layered by !grp.
oms /select tables /if subtype='Parameter Estimates' /destination numbered=REP format=sav outfile='c:\temp\fixrep.sav' /tag='fr'.
oms /select tables /if subtype='Covariance Parameter Estimates' /destination numbered=REP format=sav outfile='c:\temp\ranrep.sav' /tag='rr'.
!do !i=1 !to !nrep.
weight off.
MIXED !dep !if (!fixup=EMPTY) !then !else with !fixef !ifend
  /FIXED=INTERCEPT !if (!fixup=EMPTY) !then !else !fixef !ifend !if (!intup=NOINT) !then !else !intef !ifend
  /METHOD=ML
  /PRINT=G SOLUTION 
  /RANDOM=INTERCEPT !if (!ranup=NORAN) !then !else !ranef !ifend | SUBJECT(!level2)
  /REGWGT=!concat(!rwgt,!i).
!doend.
omsend tag='fr'.
omsend tag='rr'.
split file off.

DATASET CLOSE ALL.
get file='c:\temp\fixrep.sav'
 /keep=REP label_ !if (!grpup=NOGRP) !then Var2 !else Var1 to !concat('var',!nGrpp) !ifend Estimate.
rename var (label_ !if (!grpup=NOGRP) !then Var2 !else Var1 to !concat('var',!nGrpp) !ifend Estimate
 =EFFECT !if (!grpup=NOGRP) !then PARAMETER !else !GRPUP PARAMETER !ifend STATR).
alter type EFFECT PARAMETER (a50).
SAVE OUTFILE='C:\TEMP\1r.SAV'.

DATASET CLOSE ALL.
get file='c:\temp\ranrep.sav'
 /keep=REP label_ !if (!grpup=NOGRP) !then Var2 !else Var1 to !concat('var',!nGrpp) !ifend Estimate.
rename var (label_ !if (!grpup=NOGRP) !then Var2 !else Var1 to !concat('var',!nGrpp) !ifend Estimate
 =EFFECT !if (!grpup=NOGRP) !then PARAMETER !else !GRPUP PARAMETER !ifend STATR).
alter type EFFECT PARAMETER (a50).
SAVE OUTFILE='C:\TEMP\2r.SAV'.

aggregate /break=REP effect !if (!ngrp=0) !then !else !grp !ifend /TOTAL=sum(STATR).
select if substr(PARAMETER,1,9)='Intercept'.
compute STATR=STATR/TOTAL.
compute PARAMETER = '  Intra-class correlation'.
exe.
delete var total .

ADD FILES FILE='C:\TEMP\1r.SAV' /FILE='C:\TEMP\2r.SAV' /FILE=*.
sort cases by !if (!ngrp=0) !then !else !grp !ifend EFFECT PARAMETER.
SAVE OUTFILE=!quote(!concat('C:\TEMP\',!rwgt,'.sav')).

match files /table=!quote(!concat('C:\TEMP\',!wgt,'.sav')) /file=!quote(!concat('C:\TEMP\',!rwgt,'.sav'))
  /by !if (!ngrp=0) !then !else !grp !ifend EFFECT PARAMETER.
exe.

*** COMPUTE SAMPLING VARIANCE (U) ***.

compute var=(statr-stat)**2.
save outfile = 'c:\temp\regmod.sav'.

aggregate outfile=* /break=!if (!ngrp=0) !then !else !grp !ifend EFFECT PARAMETER /STAT= mean(stat) /var=sum(var).
compute var=!cons*var.


*** COMPUTE SE ***.

compute SE=sqrt(var).
formats stat (f8.3)/ SE (f10.6).
!if (!ngrp=0) !then compute NOGRP=1 !ifend.
save outfile='c:\temp\stats.sav'.
omsend tag="crap".


*** COMPUTE N ***.

DATASET CLOSE ALL.
get file='C:\temp\norm.sav'.
sort cases by !grp !level2.
weight OFF.
aggregate /presorted /break=!grp /NU_lev1=N.
aggregate outfile=* /presorted /break=!grp !level2 /NU_lev1=mean(NU_lev1).
aggregate outfile=* /presorted /break=!grp /NU_lev1=mean(NU_lev1) /NU_lev2=N.
match files file='c:\temp\stats.sav' /table=* /by !grp.

if (substr(parameter,1,9)='Intercept'|substr(parameter,1,8)='Residual') parameter=concat(' ',parameter).
sort cases by !if (!ngrp=0) !then !else !grp !ifend EFFECT PARAMETER.
compute PARAMETER=ltrim(PARAMETER).
EXE.

formats  NU_lev1 NU_lev2 (f8.0).
delete variables var !if (!ngrp=0) !then !grp !ifend.


*** OUTPUT TABLES ***.

oms /select tables /if subtypes='Case Processing Summary' /destination viewer=no /tag="case".
SUMMARIZE
  /TABLES= EFFECT PARAMETER !if (!nGrp>0) !then !grp !ifend stat SE NU_lev1 NU_lev2
  /FORMAT=LIST NOCASENUM
  !if (!ngrp=0) !then /TITLE=!quote(!concat("Multi-level model with ",!dep," on ",!fixup)) 
  !else /TITLE=!quote(!concat("Multi-level model with ",!dep," on ",!fixup," by ",!grp)) !ifend
  /CELLS=NONE.

save outfile="c:\temp\results.sav" / drop nu_lev1 nu_lev2.
get file="c:\temp\results.sav".

omsend tag="case".
restore.
!enddefine.