set mprint=no.

*** Created by Eveline Gebhardt, Australian Council for Educational Research ***.

define MIXED_PV   (infile = !charend('/')/
                   fixef = !default(EMPTY) !charend('/')/
                   ranef = !default(NORAN) !charend('/')/
                   intef = !default(NOINT) !charend('/')/
                   dep = !charend('/')/
                   level2 = !default(SCHOOLID) !charend('/')/
                   grp = !default(NOGRP) !charend('/')/
                   nrep = !default(80) !charend('/')/
                   wgt = !default(W_FSTUWT) !charend('/')/
                   rwgt = !default(W_FSTR) !charend('/')/
                   cons = !default(0.05) !charend('/')).

preserve.
SET Olang=English OVars names ONumbers values TVars names TNumbers values.
omsend.
oms /select headings tables texts/destination viewer=no /tag="crap".

!let !fixup=!upcase(!fixef).
!let !ranup=!upcase(!ranef).
!let !intup=!upcase(!intef).
!let !grpup=!upcase(!grp).

* nGrp is the number of group variables.
!let !nGrp=!null
!if (!upcase(!grp)=NOGRP) !then !else
!do !g !in (!grp)
!let !nGrp=!concat(!nGrp," ")
!doend !ifend.
!let !nGrpp=!length(!concat(!ngrp,!blanks(1))).
!let !nGrp=!length(!nGrp).


*** PREPARE DATA ***.

DATASET CLOSE ALL.
get file=!quote(!infile) /keep=!level2 !if (!fixup=EMPTY) !then !else !fixef !ifend !dep !if (!grpup=NOGRP) !then !else !grp !ifend
         !wgt !concat(!rwgt,'1') to !concat(!rwgt,!nrep).
!if (!grpup=NOGRP) !then compute NOGRP=1 !ifend.

* Normalise weights.
weight by !wgt.
aggregate /break=!grp /POPsize=N /SAMPsize=NU.
compute  !wgt=(!wgt/POPsize)*SAMPsize.
do repeat a= !concat(!rwgt,'1') to !concat(!rwgt,!nrep).
compute a=(a/POPsize)*SAMPsize.
end repeat.
weight off.

* Listwise deletion.
compute DEL=0.
!if (!fixup=EMPTY) !then !else
!do !f !in (!fixef).
if (missing(!f)) DEL=1.
!doend !ifend.
!do !d !in (!dep).
if (missing(!d)) DEL=1.
!doend.
select if DEL=0.
sort cases by !grp !level2.
save outfile='C:\temp\norm.sav' /drop=DEL SAMPsize POPsize.

DATASET CLOSE ALL.
get file='C:\temp\norm.sav'.


*** COMPUTE ESTIMATES ***.

split file layered by !grp.
oms /select tables /if subtype='Parameter Estimates' /destination numbered=PV format=sav outfile='c:\temp\fixful.sav' /tag='ff'.
oms /select tables /if subtype='Covariance Parameter Estimates' /destination numbered=PV format=sav outfile='c:\temp\ranful.sav' /tag='rf'.
weight off.
!do !d !in (!dep).
MIXED !d !if (!fixup=EMPTY) !then !else with !fixef !ifend
  /FIXED=INTERCEPT !if (!fixup=EMPTY) !then !else !fixef !ifend !if (!intup=NOINT) !then !else !intef !ifend
  /METHOD=ML
  /PRINT=G SOLUTION 
  /RANDOM=INTERCEPT !if (!ranup=NORAN) !then !else !ranef !ifend | SUBJECT(!level2)
  /REGWGT=!wgt.
!doend.
omsend tag='ff'.
omsend tag='rf'.
split file off.

DATASET CLOSE ALL.
get file='c:\temp\fixful.sav'
 /keep=PV label_ !if (!ngrp=0) !then Var2 !else Var1 to !concat('var',!nGrpp) !ifend Estimate.
rename var (label_ !if (!ngrp=0) !then Var2 !else Var1 to !concat('var',!nGrpp) !ifend Estimate
 =EFFECT !if (!grpup=NOGRP) !then PARAMETER !else !GRPUP PARAMETER !ifend STAT).
alter type EFFECT PARAMETER (a50).
sort cases by !if (!ngrp=0) !then !else !grp !ifend EFFECT PARAMETER PV.
casestovars /ID=!if (!ngrp=0) !then !else !grp !ifend EFFECT PARAMETER /index=PV.
SAVE OUTFILE='C:\TEMP\1.SAV'.

DATASET CLOSE ALL.
get file='c:\temp\ranful.sav'
 /keep=PV label_ !if (!grpup=NOGRP) !then Var2 !else Var1 to !concat('var',!nGrpp) !ifend Estimate.
rename var (label_ !if (!grpup=NOGRP) !then Var2 !else Var1 to !concat('var',!nGrpp) !ifend Estimate
 =EFFECT !if (!grpup=NOGRP) !then PARAMETER !else !GRPUP PARAMETER !ifend STAT).
alter type EFFECT PARAMETER (a50).
sort cases by !if (!ngrp=0) !then !else !grp !ifend EFFECT PARAMETER PV.
casestovars /ID=!if (!ngrp=0) !then !else !grp !ifend EFFECT PARAMETER /index=PV.
SAVE OUTFILE='C:\TEMP\2.SAV'.

aggregate /break=effect !if (!ngrp=0) !then !else !grp !ifend /TOTAL.1 TOTAL.2 TOTAL.3 TOTAL.4 TOTAL.5
  =sum(STAT.1 STAT.2 STAT.3 STAT.4 STAT.5).
select if substr(PARAMETER,1,9)='Intercept'.
do repeat STAT=STAT.1 STAT.2 STAT.3 STAT.4 STAT.5 /TOTAL=TOTAL.1 TOTAL.2 TOTAL.3 TOTAL.4 TOTAL.5.
compute STAT=STAT/TOTAL.
end repeat.
compute PARAMETER = '  Intra-class correlation'.
exe.
delete var TOTAL.1 TOTAL.2 TOTAL.3 TOTAL.4 TOTAL.5.
ADD FILES FILE='C:\TEMP\1.SAV' /FILE='C:\TEMP\2.SAV' /FILE=*.
sort cases by !if (!ngrp=0) !then !else !grp !ifend EFFECT PARAMETER.
SAVE OUTFILE=!quote(!concat('C:\TEMP\',!wgt,'.sav')).


*** REPLICATES ***.

DATASET CLOSE ALL.
get file='C:\temp\norm.sav'.
split file layered by !grp.
oms /select tables /if subtype='Parameter Estimates' /destination numbered=PVREP format=sav outfile='c:\temp\fixrep.sav' /tag='fr'.
oms /select tables /if subtype='Covariance Parameter Estimates' /destination numbered=PVREP format=sav outfile='c:\temp\ranrep.sav' /tag='rr'.
!do !i=1 !to !nrep.
weight off.
!do !d !in (!dep).
MIXED !d !if (!fixup=EMPTY) !then !else with !fixef !ifend
  /FIXED=INTERCEPT !if (!fixup=EMPTY) !then !else !fixef !ifend !if (!intup=NOINT) !then !else !intef !ifend
  /METHOD=ML
  /PRINT=G SOLUTION 
  /RANDOM=INTERCEPT !if (!ranup=NORAN) !then !else !ranef !ifend | SUBJECT(!level2)
  /REGWGT=!concat(!rwgt,!i).
!doend.
!doend.
omsend tag='fr'.
omsend tag='rr'.
split file off.

DATASET CLOSE ALL.
get file='c:\temp\fixrep.sav'
 /keep=PVREP label_ !if (!grpup=NOGRP) !then Var2 !else Var1 to !concat('var',!nGrpp) !ifend Estimate.
rename var (label_ !if (!ngrp=0) !then Var2 !else Var1 to !concat('var',!nGrpp) !ifend Estimate
 =EFFECT !if (!grpup=NOGRP) !then PARAMETER !else !GRPUP PARAMETER !ifend STATR).
compute PV=mod(PVREP,5).
recode PV (0=5).
if (PV=5) REP=PVREP/PV.
sort cases by !if (!ngrp=0) !then !else !grp !ifend PVREP (d).
!do !j=4 !to 1 !by -1.
if (PV=!j) REP=lag(REP).
exe.
!doend.
formats PV REP (F3.0).
sort cases by !if (!ngrp=0) !then !else !grp !ifend EFFECT PARAMETER REP PV.
delete var PVREP.
casestovars /id=!if (!ngrp=0) !then !else !grp !ifend EFFECT PARAMETER REP/index=PV.
alter type EFFECT PARAMETER (a50).
SAVE OUTFILE='C:\TEMP\1r.SAV'.

DATASET CLOSE ALL.
get file='c:\temp\ranrep.sav'
 /keep=PVREP label_ !if (!grpup=NOGRP) !then Var2 !else Var1 to !concat('var',!nGrpp) !ifend Estimate.
rename var (label_ !if (!grpup=NOGRP) !then Var2 !else Var1 to !concat('var',!nGrpp) !ifend Estimate
 =EFFECT !if (!grpup=NOGRP) !then PARAMETER !else !GRPUP PARAMETER !ifend STATR).
compute PV=mod(PVREP,5).
recode PV (0=5).
if (PV=5) REP=PVREP/PV.
sort cases by !if (!ngrp=0) !then !else !grp !ifend PVREP (d).
!do !j=4 !to 1 !by -1.
if (PV=!j) REP=lag(REP).
exe.
!doend.
formats PV REP (F3.0).
sort cases by !if (!ngrp=0) !then !else !grp !ifend EFFECT PARAMETER REP PV.
delete var PVREP.
casestovars /id=!if (!ngrp=0) !then !else !grp !ifend EFFECT PARAMETER REP/index=PV.
alter type EFFECT PARAMETER (a50).
SAVE OUTFILE='C:\TEMP\2r.SAV'.

aggregate /break=REP effect !if (!ngrp=0) !then !else !grp !ifend /TOTAL.1 TOTAL.2 TOTAL.3 TOTAL.4 TOTAL.5
  =sum(STATR.1 STATR.2 STATR.3 STATR.4 STATR.5).
select if substr(PARAMETER,1,9)='Intercept'.
do repeat STATR=STATR.1 STATR.2 STATR.3 STATR.4 STATR.5 /TOTAL=TOTAL.1 TOTAL.2 TOTAL.3 TOTAL.4 TOTAL.5.
compute STATR=STATR/TOTAL.
end repeat.
compute PARAMETER = '  Intra-class correlation'.
exe.
delete var TOTAL.1 TOTAL.2 TOTAL.3 TOTAL.4 TOTAL.5.
ADD FILES FILE='C:\TEMP\1r.SAV' /FILE='C:\TEMP\2r.SAV' /FILE=*.
sort cases by !if (!ngrp=0) !then !else !grp !ifend EFFECT PARAMETER.
SAVE OUTFILE=!quote(!concat('C:\TEMP\',!rwgt,'.sav')).

match files /table=!quote(!concat('C:\TEMP\',!wgt,'.sav')) /file=!quote(!concat('C:\TEMP\',!rwgt,'.sav'))
  /by !if (!ngrp=0) !then !else !grp !ifend EFFECT PARAMETER.
exe.


*** COMPUTE SAMPLING VARIANCE (U) ***.

do repeat a=statr.1 to statr.5/
	  b=stat.1 to stat.5/
	  c=var1 to var5.
compute c=(a-b)**2.
end repeat.
save outfile = 'c:\temp\regmod.sav'.

aggregate outfile=* /PRESORTED 
	  /break=!if (!ngrp=0) !then !else !grp !ifend EFFECT PARAMETER 
   /stat.1 to stat.5= mean(stat.1 to stat.5)
   /var1 to var5 = sum(var1 to var5).

do repeat a=var1 to var5.
compute a=!cons*a.
end repeat.

compute pv_var=mean(var1 to var5).


*** CALCULATING MEASUREMENT VARIANCE (Bm) ***.

compute STAT=mean(stat.1 to stat.5).
do repeat a=stat.1 to stat.5/b=pvar1 to pvar5.
compute b=(a-stat)**2.
end repeat.
compute pvmerr=.25*(sum(pvar1 to pvar5)).


*** COMPUTE SE [V = U + (1+1/M)Bm] ***.

compute SE=sqrt(pv_var+1.2*pvmerr).
formats stat (f8.3)/ SE (f10.6).
!if (!ngrp=0) !then compute !grp=1 !ifend.
save outfile='c:\temp\stats.sav'.

omsend tag="crap".


*** COMPUTE N ***.

DATASET CLOSE ALL.
get file='C:\temp\norm.sav'.
sort cases by !grp !level2.
weight OFF.
aggregate /presorted /break=!grp /NU_lev1=N.
aggregate outfile=* /presorted /break=!grp !level2 /NU_lev1=mean(NU_lev1).
aggregate outfile=* /presorted /break=!grp /NU_lev1=mean(NU_lev1) /NU_lev2=N.
match files file='c:\temp\stats.sav' /table=* /by !grp.

if (substr(parameter,1,9)='Intercept'|substr(parameter,1,8)='Residual') parameter=concat(' ',parameter).
sort cases by !if (!ngrp=0) !then !else !grp !ifend EFFECT PARAMETER.
compute PARAMETER=ltrim(PARAMETER).
EXE.

formats  NU_lev1 NU_lev2 (f8.0).
delete variables  stat.1 to stat.5 var1 to var5 
    pv_var pvar1 to pvar5 pvmerr !if (!ngrp=0) !then !grp !ifend.


*** OUTPUT TABLES ***.

oms /select tables /if subtypes='Case Processing Summary' /destination viewer=no /tag="case".
SUMMARIZE
  /TABLES= EFFECT PARAMETER !if (!nGrp>0) !then !grp !ifend stat SE NU_lev1 NU_lev2
  /FORMAT=LIST NOCASENUM
  !if (!ngrp=0) !then /TITLE=!quote(!concat("Multi-level model with ",!head(!dep)," to ",!substr(!dep,!Index(!dep,PV5))," on ",!fixup)) 
  !else /TITLE=!quote(!concat("Multi-level model with ",!head(!dep)," to ",!substr(!dep,!Index(!dep,PV5))," on ",!fixup," by ",!grp)) !ifend
  /CELLS=NONE.

omsend tag="case".

save outfile="c:\temp\results.sav" / drop nu_lev1 nu_lev2.
get file="c:\temp\results.sav".

restore.
!enddefine.










