set mprint=no.

*** Created by Eveline Gebhardt & Alexander Daraganov, Australian Council for Educational Research ***.

define GRPPCT (nrep = !default(80) !charend('/')/
               within = !default(NOWITHIN) !charend('/')/
               grp = !charend('/')/
               wgt = !default(W_FSTUWT) !charend('/')/
               rwgt = !default(W_FSTR) !charend('/')/
               cons = !default(0.05) !charend('/')/
               PSU = !default(SCHOOLID) !charend('/')/
               limit_criteria = !default(0) !charend('/')/
               infile = !charend('/')).

preserve.
SET Olang=English OVars names ONumbers both TVars names TNumbers both.
omsend.

/* nWithin is the number of within group variables */
!if (!upcase(!within)="NOWITHIN") !then
!let !nWithin=!null
!Else
!do !w !in (!within)
!let !nwithin=!concat(!nWithin," ")
!doend !ifend.
!let !nwithin=!length(!nwithin).

get file !infile /keep !if (!nwithin>0) !then !within !ifend !grp !PSU !wgt !concat(!rwgt,1) to !concat(!rwgt,!nrep)).
!if (!nwithin=0) !then compute !within = 1 !ifend.
sort cases by !within !grp.
save outfile='c:\temp\sort.sav'.


*** COMPUTE ESTIMATE ***.

weight by !wgt.
aggregate outfile=* mode=addvariables /presorted  /break=!within /N_all=n.
aggregate outfile=* /presorted /break=!within !grp /N_cases=n /N_all=mean(N_all) /NU_cases=NU.
formats N_cases (f10.0).
compute stat=100*(n_cases/n_all).  
save outfile=!quote(!concat('c:\temp\',!wgt,'.sav')). 


* REPLICATES.   

!do !i= 1 !to !nrep.
dataset close ALL.
get file 'c:\temp\sort.sav'.
weight by !concat(!rwgt,!i).
aggregate outfile=* mode=addvariables /presorted  /break=!within /N_all=n.
aggregate outfile=* /presorted /break=!within !grp /N_grp=n /N_all=mean(N_all).
compute statr=100*(n_grp/n_all).  
save outfile=!quote(!concat('c:\temp\',!rwgt,!i,'.sav')).
!doend.


*** COMBINE RESULTS ***.

dataset close ALL.
get file =!quote(!concat('C:\temp\',!rwgt,'1.sav')).

!let !ccc=!NULL.
!Do !e = 2 !to !nrep.
add files file=* /file=!quote(!concat('c:\temp\',!rwgt,!e,'.sav')).
exe.
!if ((!e=50) !or (!e=100) !or (!e=150) !or (!e=200) !or (!e=250) !or (!e=300) !or (!e=350) !or (!e=400) !or (!e=450)) !then
!let !ccc=(!Concat(!ccc,!Blank(1))).
save outfile=!quote(!concat('c:\temp\',!rwgt,'_temp_',!length(!ccc),'.sav')).
!let !f=!length(!concat(!blanks(!e),!blanks(1))).
get file =!quote(!concat('C:\temp\',!rwgt,!f,'.sav')).
select if $casenum=0.
!ifend
!if (!e=!nrep) !then
!let !ccc=(!Concat(!ccc,!Blank(1))).
save outfile=!quote(!concat('c:\temp\',!rwgt,'_temp_',!length(!ccc),'.sav')).
!if (!nrep>50) !then
add files !do !cccc=1 !to !length(!ccc)/file=!quote(!concat('c:\temp\',!rwgt,'_temp_',!cccc,'.sav')) !doend.
!ifend
!ifend
!Doend

sort cases by !within !grp.
match files file=* /table=!quote(!concat('c:\temp\',!wgt,'.sav')) /by !within !grp.
exec.


*** COMPUTE SAMPLING VARIANCE (U) ***.

compute var=(statr-stat)**2.
aggregate outfile=*/ break=!within !grp NU_cases N_cases/ stat= mean(stat)/ var=sum(var).
compute var=!cons*var.


*** COMPUTE SE ***.

compute SE=sqrt(var).
formats stat (f8.3)/ SE (f10.6).
sort cases by !within !grp.
save outfile='c:\temp\z.sav' /keep=!within !grp stat se NU_cases N_cases.


*** ADD NUMBER OF SAMPLED SCHOOLS ***.

get file='c:\temp\sort.sav'.
sort cases by !within !grp !PSU.
weight off.
aggregate outfile=* /break=!within !grp !PSU /N=N.
aggregate outfile=* /break=!within !grp /NU_PSU=N.
match files file='c:\temp\z.sav' /file=* /by !within !grp.


*** FLAGGING FOR OECD ***.

!if (!limit_criteria=0) !then !else
!let !z=!substr(!limit_criteria,!length(!limit_criteria),1).
!let !a=!null.
!do !b !in (!limit_criteria).
!let !a=!concat(!a," ").
!if (!length(!a)=1) !then.
compute FLAG_std=(NU_cases<!b).
formats FLAG_std (f1.0).
!ifend.
!if (!length(!a)=2) !then.
compute FLAG_sch=(NU_PSU<!b).
formats FLAG_sch (f1.0).
!ifend.
!if (!length(!a)=3) !then
aggregate /break=!if (!z=1) !then !head(!within)!else !within !ifend /N_total=sum(N_cases).
compute FLAG_pct=((100*N_cases/N_total)<!b).
formats FLAG_pct (f1.0).
!ifend.
!doend.
!ifend.
exe.


*** OUTPUT RESULTS ***.

OMS /SELECT TABLES headings
   /IF COMMANDS = ["Summarize"] SUBTYPES = ["Case Processing Summary"]
   /DESTINATION   VIEWER = NO /TAG = "SumNoCaseProc".

SUMMARIZE
  /TABLES=!if (!nwithin>0) !then !within !ifend !grp stat se !if (!limit_criteria=0) !then !else FLAG_std FLAG_sch FLAG_pct !ifend
  /FORMAT=LIST NOCASENUM NOTOTAL 
  !if (!nwithin=0) !then /TITLE=!quote(!concat('SE on group percentages of ',!grp))
  !else /TITLE=!quote(!concat('SE on group percentages of ',!grp,' within ',!within,' categories')) !ifend
  /CELLS=NONE.
OMSEND TAG=["SumNoCaseProc"].


restore.

!if (!limit_criteria=0) !then.
save outfile="c:\temp\results.sav" drop= n_cases nu_cases nu_psu.
get file="c:\temp\results.sav".
!else.
save outfile="c:\temp\results.sav" drop=n_cases nu_cases nu_psu n_total.
get file="c:\temp\results.sav".
!ifend.

!enddefine.