set mprint=no.

*** Created by Eveline Gebhardt & Alexander Daragonov, Australian Council for Educational Research ***.

*** Performance statistics by national quarters of a WLE index variable.
*** A small random value is added to the WLE index variable to make the 
*** variable continuous. This is done 5 times. Each of the new 5 continuous
*** variables is transformed into a group variable that indicate national
*** quartiles. Perfomance and index (WLE) statistics are computed within each 
*** quartile where PV1 is paired with quart1, PV2 with quart2, etc.

define PVWLEQRT (pv = !charend('/') 
                /wle = !charend('/')
                /stat = !charend('/')
                /infile = !charend('/')
                /cnt = !default(CNT) !charend('/')
                /grp = !default(NOGRP) !charend('/') 
                /psu = !default(SCHOOLID) !charend('/') 
                /wgt = !default(W_FSTUWT) !charend('/') 
                /rwgt = !default(W_FSTR) !charend('/')
                /nrep = !default(80) !charend('/')
                /cons = !default(0.05) !charend('/')
                /limit_criteria = !default(0) !charend('/')).

preserve.
SET Olang=English OVars names ONumbers values TVars names TNumbers values.
omsend.
oms /select headings tables texts/destination viewer=no /tag="crap".

/* nGrp is the number of group variables */
!if (!upcase(!grp)="NOGRP") !then
!let !nGrp=!null
!Else
!do !g !in (!grp)
!let !nGrp=!concat(!nGrp," ")
!doend !ifend.
!let !nGrp=!length(!nGrp).
title !ngrp.

get file=!quote(!infile) /keep=!cnt !psu !if (!ngrp>0) !then !grp !ifend !wgt !concat(!rwgt,1) to !concat(!rwgt,!nrep) !PV !wle.
!if (!ngrp=0) !then compute !grp=1 !ifend.
sort cases by !cnt.
weight off.
aggregate /presorted /break=!cnt /N_TOTAL=sum(!wgt).
select if not missing (!wle).


* COMPUTE CUMULATIVE COUNT WITHIN COUNTRIES.
*----------------------------------------------.
autorecode !cnt /into cnt# /print.
!do !s=1 !to 5.
set seed = !s.
!let !v=!concat(!wle,!s).
!let !c=!concat('cumfreq',!s).

compute !v=!wle+rv.normal(0,.01).
sort cases by cnt# !v.

do if ($casenum=1 or lag(cnt#) <> cnt#).
compute !c=w_fstuwt. 
else if (cnt#=lag(cnt#)).
compute !c=w_fstuwt + lag(!c). 
end if.
sort cases by !cnt.
!doend.
save outfile='C:\temp\temp.sav'.

* DEFINE CUTSCORES.
*----------------------------------------------.
weight by w_fstuwt.
aggregate /presorted /break=!cnt  /total=max(cumfreq1).
compute cut25=(total/100)*25.
compute cut50=(total/100)*50.
compute cut75=(total/100)*75.
exe.

* CREATE PERCENTILE GROUPS.
*--------------------------------------.
do repeat c=cumfreq1 cumfreq2 cumfreq3 cumfreq4 cumfreq5 /q=quart1 quart2 quart3 quart4 quart5.
if (c<cut25) q=1.
if (c>=cut25 & c<cut50) q=2.
if (c>=cut50 & c<cut75) q=3.
if (c>=cut75) q=4.
formats q (f1.0).
end repeat.
save outfile='C:\temp\quarters.sav' /keep=!cnt !psu !grp !wgt !concat(!rwgt,1) to !concat(!rwgt,!nrep) !wle !PV
    quart1 quart2 quart3 quart4 quart5 !do !i=1 !to 5 !concat(!wle,!i) !doend N_TOTAL.


*** COMPUTE STATISTIC ***.

!let !k=!null.
!do !p !in (!pv).
!let !k=!concat(!k," ").
!let !j=!length(!k).

dataset close ALL.
get file='C:\temp\quarters.sav' /keep !cnt !grp !wgt !wle !P !concat('quart',!j).
rename var (!concat('quart',!j)=!concat("Q_",!wle)).
sort cases by !cnt !grp !concat("q_",!wle).
weight by !wgt.
aggregate outfile = !quote(!concat('C:\temp\temp',!j,'.sav')) /presorted /break=!cnt !grp !concat("q_",!wle)
   /!concat('PV_STAT.',!j)=!stat(!p) /!concat('INDEX_STAT.',!j)=!stat(!wle).
!doend.    
    
match files file='C:\temp\temp1.sav'
           /file='C:\temp\temp2.sav'
           /file='C:\temp\temp3.sav'
           /file='C:\temp\temp4.sav'
           /file='C:\temp\temp5.sav'
           /by !cnt !grp !concat("q_",!wle).
save outfile=!quote(!concat('C:\temp\',!wgt,'.sav')).


*** REPLICATES ***.

!do !i= 1 !to !nrep.

!let !k=!null.
!do !p !in (!pv).
!let !k=!concat(!k," ").
!let !j=!length(!k).
*title !concat(!p," j=",!j," i=",!i).

dataset close all.
get file='C:\temp\quarters.sav' /keep !cnt !grp !concat(!rwgt,!i) !wle !P !concat('quart',!j).
rename var (!concat('quart',!j)=!concat("Q_",!wle)).
sort cases by !cnt !grp !concat("q_",!wle).
weight by !concat(!rwgt,!i).
aggregate outfile = !quote(!concat('C:\temp\temp',!j,'.sav')) /presorted /break=!cnt !grp !concat("q_",!wle)
   /!concat('PV_STATR.',!j)=!stat(!p) /!concat('INDEX_STATR.',!j)=!stat(!wle).
!doend.    
    
match files file='C:\temp\temp1.sav'
           /file='C:\temp\temp2.sav'
           /file='C:\temp\temp3.sav'
           /file='C:\temp\temp4.sav'
           /file='C:\temp\temp5.sav'
           /by !cnt !grp !concat("q_",!wle).
save outfile = !quote(!concat('C:\temp\',!rwgt,!i,'.sav')).
!doend.


*** COMBINE RESULTS ***.

dataset close ALL.
!let !nper=50.
!if ( !length(!substr(!blanks(!nrep),!length(!concat(!blanks(!nper ), !blanks(1)))))=0 ) !then
	add files file= !quote(!concat('C:\temp\',!rwgt,'1.sav')) !do !e=2 !to !nrep /file=!quote(!concat('C:\temp\',!rwgt,!e,'.sav')) !doend .
!else
	!let !modls=!nrep.
	!let !intgr=0.
	!Do !iter=!nrep !to !nper !by -!nper
		add files file= !quote(!concat('C:\temp\',!rwgt,!iter,'.sav')) !do !e=!length(!substr(!blanks(!iter),2)) !to !length(!substr(!blanks(!iter),!length(!blanks(!nper)))) !by -1 /file=!quote(!concat('C:\temp\',!rwgt,!e,'.sav')) !doend .
		!let !modls=!length(!substr(!blanks(!modls),!length(!concat(!blanks(1),!blanks(!nper))))).
		!let !intgr=!length(!Concat(!blanks(!intgr), !blanks(1))).
		save outfile=!quote(!concat('c:\temp\',!rwgt,'_temp_',!intgr,'.sav')).
	!doend
	!if (!modls>0) !then
		!if (!modls>1) !then
			add files file= !quote(!concat('C:\temp\',!rwgt,'1.sav')) !do !e=2 !to !modls /file=!quote(!concat('C:\temp\',!rwgt,!e,'.sav')) !doend .
			!let !intgr=!length(!Concat(!blanks(!intgr), !blanks(1))).
			save outfile=!quote(!concat('c:\temp\',!rwgt,'_temp_',!intgr,'.sav')).
		!else
			get file =!quote(!concat('C:\temp\',!rwgt,'1.sav')).
			!let !intgr=!length(!Concat(!blanks(!intgr), !blanks(1))).
			save outfile=!quote(!concat('c:\temp\',!rwgt,'_temp_',!intgr,'.sav')).
		!ifend
	!ifend
	!if ( !length(!substr(!blanks(!intgr),!length(!concat(!blanks(!nper ), !blanks(1)))))=0 ) !then
		add files file= !quote(!concat('C:\temp\',!rwgt,'_temp_1.sav')) !do !e=2 !to !intgr /file=!quote(!concat('C:\temp\',!rwgt,'_temp_',!e,'.sav')) !doend .
	!else
		!let !modls=!intgr.
		!let !intgr1=0.
		!do !iter=!intgr !to !nper !by -!nper
			add files file= !quote(!concat('C:\temp\',!rwgt,'_temp_',!iter,'.sav')) !do !e=!length(!substr(!blanks(!iter),2)) !to !length(!substr(!blanks(!iter),!length(!blanks(!nper)))) !ny -1 /file=!quote(!concat('C:\temp\',!rwgt,'_temp_',!e,'.sav')) !doend.
			!let !modls=!length(!substr(!blanks(!modls),!length(!concat(!blanks(1),!blanks(!nper))))).
			!let !intgr1=!length(!Concat(!blanks(!intgr1), !blanks(1))).
			save outfile=!quote(!concat('c:\temp\',!rwgt,'_tempover_',!intgr1,'.sav')).
		!doend
		!if (!modls>0) !then
			!if (!modls>1) !then
				add files file= !quote(!concat('C:\temp\',!rwgt,'_temp_1.sav')) !do !e=2 !to !modls /file=!quote(!concat('C:\temp\',!rwgt,'_temp_',!e,'.sav')) !doend .
				!let !intgr1=!length(!Concat(!blanks(!intgr1), !blanks(1))).
				save outfile=!quote(!concat('c:\temp\',!rwgt,'_tempover_',!intgr1,'.sav')).
			!else
				get file= !quote(!concat('C:\temp\',!rwgt,'_temp_1.sav')).
				!let !intgr1=!length(!Concat(!blanks(!intgr1), !blanks(1))).
				save outfile=!quote(!concat('c:\temp\',!rwgt,'_tempover_',!intgr1,'.sav')).
			!ifend
		!ifend
		add files file= !quote(!concat('c:\temp\',!rwgt,'_tempover_1.sav')) !do !e=2 !to !intgr1 /file=!quote(!concat('C:\temp\',!rwgt,'_tempover_',!e,'.sav')) !doend .
	!ifend
!ifend
exe.

sort cases by !cnt !grp !concat("q_",!wle).
match files file=*/table= !quote(!concat('C:\temp\',!wgt,'.sav'))/by !cnt !grp !concat("q_",!wle).
exec.


*** COMPUTE SAMPLING VARIANCE (U) ***.

compute INDEX_STAT=mean(INDEX_STAT.1,INDEX_STAT.2,INDEX_STAT.3,INDEX_STAT.4,INDEX_STAT.5).
compute INDEX_STATR=mean(INDEX_STATR.1,INDEX_STATR.2,INDEX_STATR.3,INDEX_STATR.4,INDEX_STATR.5).
compute INDEX_var=(INDEX_statr-INDEX_stat)**2.


do repeat a=PV_statr.1 PV_statr.2 PV_statr.3 PV_statr.4 PV_statr.5/
	  b=PV_stat.1 PV_stat.2 PV_stat.3 PV_stat.4 PV_stat.5/
	  c=PV_var1 to PV_var5.
compute c=(a-b)**2.
end repeat.

aggregate outfile=*/ break=!cnt !grp !concat("q_",!wle)/ 
          PV_stat.1 to PV_stat.5= mean(PV_stat.1 PV_stat.2 PV_stat.3 PV_stat.4 PV_stat.5)/ 
          PV_var1 to PV_var5 = sum(PV_var1 to PV_var5)/
          INDEX_STAT=mean(INDEX_STAT)/
          INDEX_VAR=sum(INDEX_VAR).

do repeat a=PV_var1 to PV_var5 INDEX_VAR.
compute a=!cons*a.
end repeat.

compute PV_VAR=mean(PV_var1 to PV_var5).


*** CALCULATING MEASUREMENT VARIANCE (Bm) ***.

compute PV_STAT=mean(PV_stat.1 to PV_stat.5).

do repeat a=PV_stat.1 to PV_stat.5/b=PV_pvar1 to PV_pvar5.
compute b=(a-PV_STAT)**2.
end repeat.

compute pvmerr=.25*(sum(PV_pvar1 to PV_pvar5)).


*** COMPUTE SE [V = U + (1+1/M)Bm] ***.

compute PV_SE=sqrt(pv_var+1.2*pvmerr).
compute INDEX_SE=sqrt(INDEX_VAR).
exe.
delete var  PV_stat.1 PV_stat.2 PV_stat.3 PV_stat.4 PV_stat.5 PV_var1 PV_var2 PV_var3 PV_var4 PV_var5
  INDEX_VAR PV_VAR PV_pvar1 PV_pvar2 PV_pvar3 PV_pvar4 PV_pvar5 pvmerr. 
formats PV_stat INDEX_STAT (f8.3)/ PV_SE INDEX_SE (f10.6).
save outfile='c:\temp\stats.sav'.
omsend tag="crap".


*** COMPUTE N ***.
!do !n=1 !to 5.
get file='C:\temp\quarters.sav' /keep= !cnt !psu !grp !wgt !concat('quart',!n) N_TOTAL.
weight off.
rename var (!concat('quart',!n)=QUART).
aggregate /break=!cnt !grp QUART /NU_CASES=N /N_CASES=sum(!wgt).
aggregate outfile=*/break=!cnt !grp QUART !psu /N_TOTAL=mean(N_TOTAL)
  /NU_CASES=mean(NU_CASES) /N_CASES=mean(N_CASES).
aggregate outfile=*/break=!cnt !grp QUART /N_TOTAL=mean(N_TOTAL)
  /NU_CASES=mean(NU_CASES) /N_CASES=mean(N_CASES) /NU_PSU=N.
save outfile=!quote(!concat('c:\temp\N',!n,'.sav')). 
!doend.

add files /file='c:\temp\N1.sav'
          /file='c:\temp\N2.sav'
          /file='c:\temp\N3.sav'
          /file='c:\temp\N4.sav'
          /file='c:\temp\N5.sav'.
aggregate outfile=* /break=!cnt !grp QUART / N_TOTAL NU_CASES N_CASES NU_PSU=
  mean(N_TOTAL NU_CASES N_CASES NU_PSU).

rename var (QUART=!concat('Q_',!wle)).
match files file='c:\temp\stats.sav' /file=* /by !cnt !grp !concat('Q_',!wle).
formats  N_TOTAL NU_CASES N_CASES NU_PSU (f10.0).
exe.

*** FLAGGING FOR OECD ***.

!if (!limit_criteria=0) !then !else
!let !a=!null.
!do !b !in (!limit_criteria).
!let !a=!concat(!a," ").
!if (!length(!a)=1) !then.
compute FLAG_std=(NU_cases<!b).
formats FLAG_std (f1.0).
!ifend.
!if (!length(!a)=2) !then
compute FLAG_sch=(NU_PSU<!b).
formats FLAG_sch (f1.0).
!ifend.
!if (!length(!a)=3) !then
compute FLAG_pct=((100*N_cases/N_total)<!b).
formats FLAG_pct (f1.0).
!ifend.
!doend.
!ifend.


*** OUTPUT RESULTS ***.

oms /select tables /if subtypes='Case Processing Summary' /destination viewer=no /tag="case".
SUMMARIZE
  /TABLES= !cnt !if (!nGrp > 0) !then !grp !ifend !concat("Q_",!wle) INDEX_STAT INDEX_SE PV_STAT PV_SE
   !if (!limit_criteria=0) !then !else FLAG_std FLAG_sch FLAG_pct !ifend
  /FORMAT=LIST NOCASENUM
  !if (!nGrp > 0) !then /TITLE=!quote(!concat("Performance by national quartiles of ",!wle," and by ",!upcase(!grp),".")) 
  !else /TITLE=!quote(!concat("Performance by national quartiles of ",!wle,".")) !ifend
  /CELLS=NONE.

omsend tag="case".
!if (!ngrp=0) !then delete var !grp !ifend.
restore.

save outfile="c:\temp\results.sav" /drop = nu_cases n_cases nu_psu n_total.
get file="c:\temp\results.sav" .

!enddefine.