set mprint=no.
define RelRisk (outcome = !charend('/')/
                antecedent = !charend('/')/
                infile = !charend('/')/
                grp = !default(NOGRP) !charend('/')/
                wgt = !default(W_FSTUWT) !charend('/')/
                rwgt = !default(W_FSTR) !charend('/')/
                cons = !default(0.05) !charend('/')/
                nrep = !default(80) !charend('/')).
                
preserve.
SET Olang=English OVars names ONumbers values TVars names TNumbers values.
omsend.
oms /select headings tables texts/destination viewer=no /tag="crap".

/* nOut is the number of outcome variables (1 or 5) */
!let !nOut=!null.
!do !o !in (!outcome)
!let !nOut=!concat(!nOut,!blanks(1)).
!doend.
!let !nOut=!length(!nOut).
title Number of outcome variables= !nOut.

/* nAnt is the number of antecedent variable (1 or 5) */
!let !nAnt=!null.
!do !o !in (!antecedent)
!let !nAnt=!concat(!nAnt,!blanks(1)).
!doend.
!let !nAnt=!length(!nAnt).
title Number of antecedent variables= !nAnt.

/* nGrp is the number of group variables */
!let !nGrp=!null.
!do !g !in (!grp)
!let !nGrp=!concat(!nGrp,!blanks(1))
!doend.
!let !nGrp=!length(!nGrp).
!if (!upcase(!grp)=NOGRP) !then !let !ngrp=0 !ifend.
title Number of group variables = !ngrp.


*** COMPUTE RELATIVE RISK ***.

get file !quote(!infile) /keep !if (!ngrp=0) !then !else !grp !ifend !outcome !antecedent !wgt !concat(!rwgt,1) to !concat(!rwgt,!nrep) .
!if (!ngrp=0) !then compute !grp=1 !ifend.

!if (!nOut=1 & !nAnt=1) !then
select if (not missing(!antecedent)).
select if (not missing(!outcome)).
weight by !wgt.
aggregate outfile=* /break=!GRP !outcome !antecedent /n=N.
!ifend
exe.
sort cases by !grp !antecedent !outcome.
casestovars /id=!grp /index=!antecedent !outcome.
compute STAT=(N.1.1/(N.1.0+N.1.1))/(N.0.1/(N.0.0+N.0.1)).
save outfile=!quote(!concat('c:\temp\',!wgt,'.sav')) /keep=!grp STAT.


*** REPLICATION ***.

!do !i=1 !to !nrep.
get file !quote(!infile) /keep !if (!ngrp=0) !then !else !grp !ifend !outcome !antecedent !wgt !concat(!rwgt,1) to !concat(!rwgt,!nrep) .
!if (!ngrp=0) !then compute !grp=1 !ifend.

!if (!nOut=1 & !nAnt=1) !then
select if (not missing(!antecedent)).
select if (not missing(!outcome)).
weight by !concat(!rwgt,!i).
aggregate outfile=* /break=!GRP !outcome !antecedent /n=N.
!ifend
exe.
sort cases by !grp !antecedent !outcome.
casestovars /id=!grp /index=!antecedent !outcome.
compute STATR=(N.1.1/(N.1.0+N.1.1))/(N.0.1/(N.0.0+N.0.1)).
save outfile=!quote(!concat('c:\temp\',!rwgt,!i,'.sav')) /keep=!grp STATR.
!doend.


* Combine replicate files.

!let !nper=50.
!if ( !length(!substr(!blanks(!nrep),!length(!concat(!blanks(!nper ),!blanks(1)))))=0 ) !then
	add files file= !quote(!concat('C:\temp\',!rwgt,'1.sav')) !do !e=2 !to !nrep /file=!quote(!concat('C:\temp\',!rwgt,!e,'.sav')) !doend .
!else
	!let !modls=!nrep.
	!let !intgr=0.
	!Do !iter=!nrep !to !nper !by -!nper
		add files file= !quote(!concat('C:\temp\',!rwgt,!iter,'.sav')) !do !e=!length(!substr(!blanks(!iter),2)) !to !length(!substr(!blanks(!iter),!length(!blanks(!nper)))) !by -1 /file=!quote(!concat('C:\temp\',!rwgt,!e,'.sav')) !doend .
		!let !modls=!length(!substr(!blanks(!modls),!length(!concat(!blanks(1),!blanks(!nper))))).
		!let !intgr=!length(!Concat(!blanks(!intgr), !blanks(1))).
		save outfile=!quote(!concat('c:\temp\',!rwgt,'_temp_',!intgr,'.sav')).
	!doend
	!if (!modls>0) !then
		!if (!modls>1) !then
			add files file= !quote(!concat('C:\temp\',!rwgt,'1.sav')) !do !e=2 !to !modls /file=!quote(!concat('C:\temp\',!rwgt,!e,'.sav')) !doend .
			!let !intgr=!length(!Concat(!blanks(!intgr), !blanks(1))).
			save outfile=!quote(!concat('c:\temp\',!rwgt,'_temp_',!intgr,'.sav')).
		!else
			get file =!quote(!concat('C:\temp\',!rwgt,'1.sav')).
			!let !intgr=!length(!Concat(!blanks(!intgr), !blanks(1))).
			save outfile=!quote(!concat('c:\temp\',!rwgt,'_temp_',!intgr,'.sav')).
		!ifend
	!ifend
	!if ( !length(!substr(!blanks(!intgr),!length(!concat(!blanks(!nper ), !blanks(1)))))=0 ) !then
		add files file= !quote(!concat('C:\temp\',!rwgt,'_temp_1.sav')) !do !e=2 !to !intgr /file=!quote(!concat('C:\temp\',!rwgt,'_temp_',!e,'.sav')) !doend .
	!else
		!let !modls=!intgr.
		!let !intgr1=0.
		!do !iter=!intgr !to !nper !by -!nper
			add files file= !quote(!concat('C:\temp\',!rwgt,'_temp_',!iter,'.sav')) !do !e=!length(!substr(!blanks(!iter),2)) !to !length(!substr(!blanks(!iter),!length(!blanks(!nper)))) !ny -1 /file=!quote(!concat('C:\temp\',!rwgt,'_temp_',!e,'.sav')) !doend.
			!let !modls=!length(!substr(!blanks(!modls),!length(!concat(!blanks(1),!blanks(!nper))))).
			!let !intgr1=!length(!Concat(!blanks(!intgr1), !blanks(1))).
			save outfile=!quote(!concat('c:\temp\',!rwgt,'_tempover_',!intgr1,'.sav')).
		!doend
		!if (!modls>0) !then
			!if (!modls>1) !then
				add files file= !quote(!concat('C:\temp\',!rwgt,'_temp_1.sav')) !do !e=2 !to !modls /file=!quote(!concat('C:\temp\',!rwgt,'_temp_',!e,'.sav')) !doend .
				!let !intgr1=!length(!Concat(!blanks(!intgr1), !blanks(1))).
				save outfile=!quote(!concat('c:\temp\',!rwgt,'_tempover_',!intgr1,'.sav')).
			!else
				get file= !quote(!concat('C:\temp\',!rwgt,'_temp_1.sav')).
				!let !intgr1=!length(!Concat(!blanks(!intgr1), !blanks(1))).
				save outfile=!quote(!concat('c:\temp\',!rwgt,'_tempover_',!intgr1,'.sav')).
			!ifend
		!ifend
		add files file= !quote(!concat('c:\temp\',!rwgt,'_tempover_1.sav')) !do !e=2 !to !intgr1 /file=!quote(!concat('C:\temp\',!rwgt,'_tempover_',!e,'.sav')) !doend .
	!ifend
!ifend
exe.
sort cases by !grp.
save outfile='c:\temp\rep.sav'.

match files /table=!quote(!concat('c:\temp\',!wgt,'.sav')) /file='c:\temp\rep.sav' /by !grp.


*** COMPUTE SAMPLING VARIANCE (U) ***.

compute var=(statr-stat)**2.
aggregate outfile=* /break=!grp /stat= mean(stat) /var=sum(var).
compute var=!cons*var.


*** COMPUTE SE ***.

compute se=sqrt(var).
formats stat (f8.3)/ SE (f10.6).
save outfile='c:\temp\stats.sav'.

omsend tag="crap".


*** OUTPUT TABLES ***.

oms /select tables /if subtypes='Case Processing Summary' /destination viewer=no /tag="case".
SUMMARIZE
  /TABLES= !if (!nGrp>0) !then !grp !ifend stat SE
  /FORMAT=LIST NOCASENUM
  !if (!ngrp=0) !then /TITLE=!quote(!concat("Relative risk with ",!antecedent,' as antecedent and ',!outcome,' as outcome variable')))) 
  !else /TITLE=!quote(!concat("Relative risk with ",!antecedent,' as antecedent and ',!outcome,' as outcome variable by ',!grp)) !ifend
  /CELLS=NONE.

delete variables !if (!ngrp=0) !then !grp !ifend  var.
omsend tag="case".
restore.

save outfile="c:\temp\results.sav".


!enddefine.

