set mprint=no.

*** Created by Eveline Gebhardt, Australian Council for Educational Research ***.

define CORnoPV (var1 = !charend('/') /
                var2 = !charend('/') /
                infile = !charend('/')/
                grp = !default(NOGRP) !charend('/')  /
                limit_criteria = !default(0) !charend('/')/
                PSU = !default(SCHOOLID) !charend('/')/
                wgt = !default(W_FSTUWT) !charend('/') /
                rwgt = !default(W_FSTR) !charend('/') /
                nrep = !default(80) !charend('/')/
                cons = !default(0.05) !charend('/')).

preserve.
SET Olang=English OVars names ONumbers values TVars names TNumbers values.
omsend.
oms /select tables headings texts/destination viewer=no /tag="NoViewer".

/* nGrp is the number of group variables */
!if (!upcase(!grp)="NOGRP") !then
!let !nGrp=!null
!Else
!do !g !in (!grp)
!let !nGrp=!concat(!nGrp," ")
!doend !ifend.
!let !nGrp=!length(!nGrp).

!if (!nGrp>0) !then 
!let !x=!length(!concat(!blanks(!nGrp)," ")).
!let !y=!length(!concat(!blanks(!nGrp),"  ")).
!else 
!let !x=2.
!let !y=3.
!ifend.
title grp=!grp ngrp=!ngrp x=!x y=!y.

get file !infile /keep !if (!ngrp>0) !then !grp !ifend !PSU !wgt !concat(!rwgt,1) to !concat(!rwgt,!nrep) !var1 !var2.
!if (!nGrp=0) !then compute !grp=1 !ifend.
missing values !grp ().
SORT CASES by !grp.
compute MISS = (missing(!var1)).
if (missing(!var2)) MISS=1.
compute VALIDN=1-MISS.
formats VALIDN (f1.0).
save outfile='c:\temp\sort.sav' /drop=MISS.


*** COMPUTE ESTIMATE ***.

oms select tables /if subtypes='Correlations' /destination format=sav outfile='c:\temp\full.sav' /tag="fullweight".
split file layered by !grp.
weight by !wgt.
CORRELATIONS  /VARIABLES=!var1 with !var2
  /PRINT=TWOTAIL NOSIG .
split file off.
omsend tag="fullweight".

get file='c:\temp\full.sav' /keep= Var1 to !concat('Var',!y) !var2.
rename vars (!var2=STAT).
compute casenum=$casenum.
exe.
select if mod(casenum,3)=1.
exe.
*erase file='c:\temp\fullwgt.sav'.
!if (!nGrp > 0) !then 
sort cases by Var1 to !concat('Var',!nGrp) 
!ifend.
save outfile='c:\temp\fullwgt.sav' /drop= !concat('Var',!x) !concat('Var',!y).


* REPLICATES.
get file='c:\temp\sort.sav'.

oms select tables /if subtypes='Correlations' /destination numbered="RepWeight" format=sav outfile='c:\temp\rep.sav' /tag="repweight".

!do !i=1 !to !nrep.
split file layered by !grp.
weight by !concat(!rwgt,!i).
CORRELATIONS /VARIABLES=!var1 with !var2
  /PRINT=TWOTAIL NOSIG /MISSING=PAIRWISE .
!doend.
omsend tag="RepWeight".

get file='c:\temp\rep.sav' /keep=RepWeight Var1 to !concat('Var',!y) !var2.
rename vars (!var2=statR).
compute casenum=$casenum.
exe.
select if mod(casenum,3)=1.
exe.
!if (!nGrp > 0) !then 
sort cases by Var1 to !concat('Var',!nGrp) 
!ifend.
save outfile='c:\temp\repwgt.sav' /drop= !concat('Var',!x) !concat('Var',!y) .


*** COMBINE RESULTS ***.

match files table='c:\temp\fullwgt.sav' /file='c:\temp\repwgt.sav' /by !if (!nGrp>1) !then Var1 to !concat('Var',!nGrp) !else Var1 !ifend.
string Correlation (a100).
compute Correlation=!quote(!concat(!var1," * ",!var2)).
rename vars (!if (!nGrp>1) !then Var1 to !concat('Var',!nGrp) !else Var1 !ifend  = !grp ).
exec.


*** COMPUTE SAMPLING VARIANCE (U) ***.

compute var=(statr-stat)**2.
save outfile = 'c:\temp\regmod.sav'.

aggregate outfile=*/ break=!grp/ stat= mean(stat)/ var=sum(var).
compute var=!cons*var.


*** COMPUTE SE ***.

compute SE=sqrt(var).
formats stat (f8.3)/ SE (f10.6).
save outfile='c:\temp\stats.sav'.


*** COMPUTE N ***.

!let !z=!substr(!limit_criteria,!length(!limit_criteria),1).
get file='c:\temp\sort.sav'.
weight by !wgt.
!if (!limit_criteria=0) !then !else
aggregate /break=!if (!z=1) !then !head(!grp) !else !grp !ifend /N_total=N.
!ifend
filter by VALIDN.
aggregate outfile='c:\temp\y.sav' /break=!grp /NU_cases=NU /N_cases=N.
weight off.
aggregate outfile=* /break=!grp !PSU /N=N !if (!limit_criteria=0) !then !else /N_total=mean(N_total) !ifend.
aggregate outfile=* /break=!grp /NU_PSU=N !if (!limit_criteria=0) !then !else /N_total=mean(N_total) !ifend.
match files /file='c:\temp\y.sav' /file=* /by !grp.
match files file='c:\temp\stats.sav' /TABLE=* /by !grp.
formats N_cases (f10.0).
filter off.


*** FLAGGING FOR OECD ***.

!if (!limit_criteria=0) !then !else
!let !a=!null.
!do !b !in (!limit_criteria).
!let !a=!concat(!a," ").
!if (!length(!a)=1) !then.
compute FLAG_std=(NU_cases<!b).
formats FLAG_std (f1.0).
!ifend.
!if (!length(!a)=2) !then.
compute FLAG_sch=(NU_PSU<!b).
formats FLAG_sch (f1.0).
!ifend.
!if (!length(!a)=3) !then
compute FLAG_pct=((100*N_cases/N_total)<!b).
formats FLAG_pct (f1.0).
!ifend.
!doend.
!ifend.
exe.
omsend tag="NoViewer".


*** OUTPUT TABLES ***.

oms /select tables /if subtypes='Case Processing Summary' /destination viewer=no /tag="case".
SUMMARIZE
  /TABLES= !if (!nGrp > 0) !then !grp !ifend stat SE !if (!limit_criteria=0) !then !else FLAG_std FLAG_sch FLAG_pct !ifend
  /FORMAT=LIST NOCASENUM
  !if (!ngrp=0) !then /TITLE=!quote(!concat("Correlation ",!var1," with ",!var2)) 
  !else /TITLE=!quote(!concat("Correlation ",!var1," with ",!var2," by ",!grp)) !ifend
  /CELLS=NONE.

omsend tag="case".

delete variables var !if (!ngrp=0) !then !grp !ifend.
restore.

!if (!limit_criteria=0) !then.
save outfile="c:\temp\results.sav" drop= n_cases nu_cases nu_psu.
get file="c:\temp\results.sav".
!else.
save outfile="c:\temp\results.sav" drop=n_cases nu_cases nu_psu n_total.
get file="c:\temp\results.sav".
!ifend.

!enddefine.