set mprint=no.

*** Created by Eveline Gebhardt, Australian Council for Educational Research ***.

define COR_2PV (PV1 = !charend('/') /
                PV2 = !charend('/') /
                infile = !charend('/')/
                grp = !default(NOGRP) !charend('/')  /
                limit_criteria = !default(0) !charend('/')/
                PSU = !default(SCHOOLID) !charend('/')/
                wgt = !default(W_FSTUWT) !charend('/') /
                rwgt = !default(W_FSTR) !charend('/') /
                nrep = !default(80) !charend('/')/
                cons = !default(0.05) !charend('/')).
preserve.
SET Olang=English OVars names ONumbers values TVars names TNumbers values.
omsend.
oms /select tables headings texts/destination viewer=no /tag="NoViewer".

/* nGrp is the number of group variables */
!if (!upcase(!grp)="NOGRP") !then
!let !nGrp=!null
!Else
!do !g !in (!grp)
!let !nGrp=!concat(!nGrp," ")
!doend !ifend.
!let !nGrp=!length(!nGrp).

!if (!nGrp>0) !then 
!let !x=!length(!concat(!blanks(!nGrp)," ")).
!let !y=!length(!concat(!blanks(!nGrp),"  ")).
!else 
!let !x=2.
!let !y=3.
!ifend.
title grp=!grp ngrp=!ngrp x=!x y=!y.


get file !infile /keep !if (!ngrp>0) !then !grp !ifend !PSU !wgt !concat(!rwgt,1) to !concat(!rwgt,!nrep) !PV1 !PV2.
!if (!nGrp=0) !then compute !grp=1 !ifend.
missing values !grp ().
SORT CASES by !grp.
compute MISS = 0.
!do !p1 !in (!pv1).
if (missing(!p1)) MISS=1.
!doend.
!do !p2 !in (!pv2).
if (missing(!p2)) MISS=1.
!doend.
compute VALIDN=1-MISS.
formats VALIDN (f1.0).
save outfile='c:\temp\sort.sav' /drop=MISS.


*** COMPUTE ESTIMATE ***.

oms select tables /if subtypes='Correlations' /destination format=sav outfile='c:\temp\full.sav' /tag="fullweight".
split file layered by !grp.
weight by !wgt.
CORRELATIONS  /VARIABLES=!PV2 with !PV1
  /PRINT=TWOTAIL NOSIG /MISSING=PAIRWISE .
split file off.
omsend tag="fullweight".

get file='c:\temp\full.sav'  /keep= Var1 to !concat('Var',!y) !PV1.
compute casenum=$casenum.
exe.
select if mod(casenum,3)=1.
exe.
rename vars (!pv1=stat1 to stat5).

!do !p=1 !to 5.
if (index(!concat('var',!x),!quote(!concat("PV",!p)))=0 & index(!concat('var',!x),!quote(!concat("pv",!p)))=0) !concat("stat",!p)=$sysmis.
!doend.
exe.
aggregate /outfile=* /break=!if (!nGrp>1) !then Var1 to !concat('Var',!nGrp) !else Var1 !ifend /stat1 to stat5=mean(stat1 to stat5).
!if (!nGrp > 0) !then 
sort cases by Var1 to !concat('Var',!nGrp) .
!ifend.

*erase file='c:\temp\fullwgt.sav'.
save outfile='c:\temp\fullwgt.sav'.


* REPLICATES.

get file='c:\temp\sort.sav'.

oms select tables /if subtypes='Correlations' /destination numbered="RepWeight" format=sav outfile='c:\temp\rep.sav' /tag="repweight".

!do !i=1 !to !nrep.
split file layered by !grp.
weight by !concat(!rwgt,!i).
CORRELATIONS /VARIABLES=!PV2 with !PV1
  /PRINT=TWOTAIL NOSIG /MISSING=PAIRWISE .
!doend.
omsend tag="RepWeight".

get file='c:\temp\rep.sav' /keep=RepWeight Var1 to !concat('Var',!y) !PV1.
compute casenum=$casenum.
exe.
select if mod(casenum,3)=1.
exe.
rename vars (!pv1=statR1 to statR5).

!do !p=1 !to 5.
if (index(!concat('var',!x),!quote(!concat("PV",!p)))=0 & index(!concat('var',!x),!quote(!concat("pv",!p)))=0) !concat("statR",!p)=$sysmis.
!doend.
exe.
aggregate /outfile=* /break=!if (!nGrp>1) !then Var1 to !concat('Var',!nGrp) !else Var1 !ifend RepWeight /statR1 to statR5=mean(statR1 to statR5).
!if (!nGrp > 0) !then 
sort cases by Var1 to !concat('Var',!nGrp) .
!ifend.
save outfile='c:\temp\repwgt.sav' .


*** COMBINE RESULTS ***.

match files table='c:\temp\fullwgt.sav' /file='c:\temp\repwgt.sav' /by !if (!nGrp>1) !then Var1 to !concat('Var',!nGrp) !else Var1 !ifend .
string Correlation (a100).
compute Correlation=!quote(!concat(!PV2," * ",!PV1)).
rename vars (!if (!nGrp>1) !then Var1 to !concat('Var',!nGrp) !else Var1 !ifend  = !grp ).
exec.


*** COMPUTE SAMPLING VARIANCE (U) ***.

do repeat a=statr1 to statr5/
	  b=stat1 to stat5/
	  c=var1 to var5.
compute c=(a-b)**2.
end repeat.
save outfile = 'c:\temp\regmod.sav'.

aggregate outfile=*/ break=!grp/ stat1 to stat5= mean(stat1 to stat5)/ var1 to var5 = sum(var1 to var5).
do repeat a=var1 to var5.
compute a=!cons*a.
end repeat.
compute pv_var=mean(var1 to var5).


*** CALCULATING MEASUREMENT VARIANCE (Bm) ***.

compute stat=mean(stat1 to stat5).
do repeat a=stat1 to stat5/b=pvar1 to pvar5.
compute b=(a-stat)**2.
end repeat.
compute pvmerr=.25*(sum(pvar1 to pvar5)).


*** COMPUTE SE [V = U + (1+1/M)Bm] ***.

compute SE=sqrt(pv_var+1.2*pvmerr).
formats stat (f8.3)/ SE (f10.6).
save outfile='c:\temp\stats.sav'.


*** COMPUTE N ***.

!let !z=!substr(!limit_criteria,!length(!limit_criteria),1).
get file='c:\temp\sort.sav'.
weight by !wgt.
!if (!limit_criteria=0) !then !else
aggregate /break=!if (!z=1) !then !head(!grp) !else !grp !ifend /N_total=N.
!ifend
filter by VALIDN.
aggregate outfile='c:\temp\y.sav' /break=!grp /NU_cases=NU /N_cases=N.
weight off.
aggregate outfile=* /break=!grp !PSU /N=N !if (!limit_criteria=0) !then !else /N_total=mean(N_total) !ifend.
aggregate outfile=* /break=!grp /NU_PSU=N !if (!limit_criteria=0) !then !else /N_total=mean(N_total) !ifend.
match files /file='c:\temp\y.sav' /file=* /by !grp.
match files file='c:\temp\stats.sav' /TABLE=* /by !grp.
formats N_cases (f10.0).
filter off.


*** FLAGGING FOR OECD ***.

!if (!limit_criteria=0) !then !else
!let !a=!null.
!do !b !in (!limit_criteria).
!let !a=!concat(!a," ").
!if (!length(!a)=1) !then.
compute FLAG_std=(NU_cases<!b).
formats FLAG_std (f1.0).
!ifend.
!if (!length(!a)=2) !then.
compute FLAG_sch=(NU_PSU<!b).
formats FLAG_sch (f1.0).
!ifend.
!if (!length(!a)=3) !then
compute FLAG_pct=((100*N_cases/N_total)<!b).
formats FLAG_pct (f1.0).
!ifend.
!doend.
!ifend.
exe.
omsend tag="NoViewer".


*** OUTPUT TABLES ***.

oms /select tables /if subtypes='Case Processing Summary' /destination viewer=no /tag="case".
SUMMARIZE
  /TABLES= !if (!nGrp > 0) !then !grp !ifend stat SE !if (!limit_criteria=0) !then !else FLAG_std FLAG_sch FLAG_pct !ifend
  /FORMAT= LIST NOCASENUM
  !if (!ngrp=0) !then /TITLE=!quote(!concat("Correlation ",!concat(!head(!PV1)," to ",!substr(!PV1,!Index(!PV1,PV5))," with ",!head(!PV2)," to ",!substr(!PV2,!Index(!PV2,PV5))))) 
  !else /TITLE=!quote(!concat("Correlation ",!concat(!head(!PV1)," to ",!substr(!PV1,!Index(!PV1,PV5))," with ",!head(!PV2)," to ",!substr(!PV2,!Index(!PV2,PV5)))," by ",!grp)) !ifend
  /CELLS=NONE.

omsend tag="case".

delete variables !if (!ngrp=0) !then !grp !ifend stat1 stat2 stat3 stat4 stat5 var1 var2 var3 var4 var5 pv_var pvar1 pvar2 pvar3 pvar4 pvar5 pvmerr.
restore.

!if (!limit_criteria=0) !then.
save outfile="c:\temp\results.sav" drop= n_cases nu_cases nu_psu.
get file="c:\temp\results.sav".
!else.
save outfile="c:\temp\results.sav" drop=n_cases nu_cases nu_psu n_total.
get file="c:\temp\results.sav".
!ifend.

!enddefine.
