set mprint=no.

*** Created by Eveline Gebhardt, Australian Council for Educational Research ***.

*** !!! THE CATEGORIES OF THE 'COMPARE' VARIABLE SHOULD BE NUMBERS, NOT LETTERS OR WORDS ***.

define DIFnoPV (dep = !charend('/') /
                stat = !charend ('/') /
                compare = !charend('/') /
                categ = !charend ('/') /
                infile = !charend('/')/
                within = !default(NOWITHIN) !charend('/') /
                wgt = !default(W_FSTUWT) !charend('/') /
                rwgt = !default(W_FSTR) !charend('/') /
                cons = !default(0.05) !charend('/')/
                nrep = !default(80) !charend('/')).

preserve.
SET Olang=English OVars names ONumbers values TVars names TNumbers values.
omsend.

/* nWithin is the number of within group variables */
!if (!upcase(!within)="NOWITHIN") !then
!let !nWithin=!null
!Else
!do !w !in (!within)
!let !nwithin=!concat(!nWithin," ")
!doend !ifend.
!let !nwithin=!length(!nwithin).

/* z is nWithin + 1 */
!if (!nWithin>0) !then
!let !z=!length(!concat(!blanks(!nWithin)," ")).
!else
!let !z=2.
!ifend.
*title !concat('number of Within group variables is ',!nWithin,', plus 1 (z) = ',!z).

/* ncat is the number of categories
!let !nc=!null.
!do !c !in (!categ) !let !nc=!concat(!nc," ") !doend.
!let !ncat=!length(!nc).
*title !concat("Number of categories=",!ncat).

get file !quote(!infile) /keep !dep !if (!nwithin>0) !then !within !ifend !compare !wgt !concat(!rwgt,1) to !concat(!rwgt,!nrep).
!if (!nwithin=0) !then compute !within=1 !ifend.
compute MISS = (missing(!dep)).
if (missing(!compare)) MISS=1.
!do !w !in (!within).
if (missing(!w)) MISS=1. 
!doend.
!let !n=!null.
!do !c !in(!categ)
!let !n=!concat(!n,'| ',!compare,'=',!c,!blanks(1))
!doend.
!let !n=!concat('(',!substr(!n,2),')').
*title !n.
compute VAL=!n.
if (VAL=0) MISS=1.
compute VALIDN=1-MISS.
formats VALIDN (f1.0).
save outfile='c:\temp\data.sav' /drop=MISS.

*------------ Next table with counts does not work without licence for CLABELS ---------.
*!let !q=!null.
*!if (!nwithin>0) !then
*!do !w !in(!within).
*!let !q=!concat(!q,!w,' [C] > ').
*!doend !ifend.
**title !q.
*
*VARIABLE LEVEL !within !compare (NOMINAL).
*string INCLUDED (a3).
*if (validn=0) INCLUDED='No'.
*if (validn=1) INCLUDED='Yes'.
*weight by !wgt.
*CTABLES
*  /VLABELS VARIABLES=!if (!nwithin>0) !then !within !ifend !compare INCLUDED DISPLAY=DEFAULT
*  /TABLE !q !compare [C][UCOUNT F40.0, COUNT F40.0] BY INCLUDED [C]
*  /CATEGORIES VARIABLES=!if (!nwithin>0) !then !within !ifend !compare INCLUDED ORDER=A KEY=VALUE EMPTY=EXCLUDE MISSING=INCLUDE
*  /TITLES TITLE='Columns "INCLUDED=Yes" give the weighted and unweighted number of included students'.
*---------------------------------------------------------------------------.
oms /select tables headings texts/destination viewer=no /tag="NoViewer".


*** COMPUTE ESTIMATES ***.

oms /select tables /if subtypes='Report' /destination format=sav outfile=!quote(!concat('c:\temp\',!wgt,'_1.sav')) /tag="fullweight".
select if (VALIDN=1).
weight by !wgt.
means !dep !if (!nWithin>0) !then !do !w !in (!within) !concat("by ",!w) !doend !else by !within !ifend by !compare /cells=!stat.
omsend tag="fullweight".

Get file=!quote(!concat('c:\temp\',!wgt,'_1.sav')) /keep= Var1 to !concat('var',!z) !dep .
rename vars (Var1 to !concat('var',!z) = !within !compare).

compute y=number(!compare,f3.0).
select if (not missing(y)).
compute !compare=concat('cat',!compare).
sort cases by !within !compare.
delete var y.
casestovars /id=!within /index=!compare.

/* This part computes the difference between each pair of categories */. 
*--------------------------------------------------------------------------------------------------------.
!let !vars=!null.
!let !a=!categ.
!do !x !in (!categ)
!do !c !in (!tail(!a)) 
!if (!tail(!a) > !null) !then
compute !concat(!compare,".",!x,"_",!c)=!concat("cat",!x) - !concat("cat",!c).
!let !vars=!concat(!vars,!concat(!compare,".",!x,"_",!c)," ").
!ifend !doend
!let !a=!tail(!a).
!doend.
exe.
*--------------------------------------------------------------------------------------------------------.

delete variables !do !c !in (!categ) !concat("cat",!c) !doend.
VARSTOCASES  /MAKE stat FROM !vars
 /INDEX = DIFF(stat)
 /KEEP =  !within
 /NULL = KEEP.
sort cases by !within DIFF.
save outile=!quote(!concat('c:\temp\',!wgt,'.sav')).


*** REPLICATES ***.

dataset close all.
get file='c:\temp\data.sav'.
select if (VALIDN=1).
oms /select tables /if subtypes='Report' /destination format=sav numbered='REP' outfile=!quote(!concat('c:\temp\',!rwgt,'_1.sav')) /tag="r".
!do !i=1 !to !nrep.
weight by !concat(!rwgt,!i).
means !dep !if (!nWithin>0) !then !do !w !in (!within) !concat("by ",!w) !doend !else by !within !ifend by !compare /cells=!stat.
!doend.
omsend tag="r".

get file=!quote(!concat('c:\temp\',!rwgt,'_1.sav')) /keep= rep Var1 to !concat('var',!z) !dep .
rename vars (Var1 to !concat('var',!z) = !within !compare).
compute y=number(!compare,f3.0).
select if (not missing(y)).
compute !compare=concat('cat',!compare).
sort cases by rep !within !compare.
delete var y.
casestovars /id=rep !within /index=!compare.

/* This part computes the difference between each pair of categories */. 
*--------------------------------------------------------------------------------------------------------.
!let !vars=!null.
!let !a=!categ.
!do !x !in (!categ)
!do !c !in (!tail(!a)) 
!if (!tail(!a) > !null) !then
compute !concat(!compare,".",!x,"_",!c)=!concat("cat",!x) - !concat("cat",!c).
!let !vars=!concat(!vars,!concat(!compare,".",!x,"_",!c)," ").
!ifend !doend
!let !a=!tail(!a).
!doend.
exe.
*--------------------------------------------------------------------------------------------------------.
delete variables !do !c !in (!categ) !concat("cat",!c) !doend.
VARSTOCASES  /MAKE statR FROM !vars
 /INDEX = DIFF(statR)
 /KEEP =  rep !within
 /NULL = KEEP.
sort cases by !within DIFF.


*** COMBINE RESULTS ***.

match files table=!quote(!concat('c:\temp\',!wgt,'.sav')) /file=* /by !within DIFF.


*** COMPUTE SAMPLING VARIANCE (U) ***.

compute var=(statr-stat)**2.
aggregate outfile=*/ break !within diff/ stat= mean(stat)/ var=sum(var).
compute var=!cons*var.


*** COMPUTE SE ***.

compute se=sqrt(var).
exe.

formats stat (f8.3)/ SE (f10.6).
omsend tag="NoViewer".

*** OUTPUT TABLE ***.

OMS /SELECT TABLES headings
   /IF COMMANDS = ["Summarize"] SUBTYPES = ["Case Processing Summary"]
   /DESTINATION   VIEWER = NO /TAG = "SumNoCaseProc".

SUMMARIZE
  /TABLES=!if (!nWithin>0) !then !within !ifend diff stat se
  /FORMAT=LIST NOCASENUM NOTOTAL 
  !if (!nwithin=0) !then /TITLE=!quote(!concat('SE on difference in ',!upcase(!dep),' between ',!upcase(!compare),' group ',!upcase(!stat),'s'))
  !else /TITLE=!quote(!concat('SE on difference in ',!upcase(!dep),' between ',!upcase(!compare),' group ',!upcase(!stat),'s by ',!upcase(!within))) !ifend
  /CELLS=NONE.
OMSEND TAG=["SumNoCaseProc"].

delete variables var !if (!nWithin=0) !then !within !ifend.

restore.

save outfile="c:\temp\results.sav".

!enddefine.