set mprint=no.

*** Created by Eveline Gebhardt, Australian Council for Educational Research ***.

*** !!! THE 'COMPARE' VARIABLE SHOULD BE NUMERIC, NOT STRING ***.

define EFF_PV  (dep = !charend('/') /
                compare = !charend('/') /
                categ = !charend ('/') /
                infile = !charend('/')/
                within = !default(NOWITHIN) !charend('/') /
                wgt = !default(W_FSTUWT) !charend('/') /
                rwgt = !default(W_FSTR) !charend('/') /
                cons = !default(0.05) !charend('/')/
                nrep = !default(80) !charend('/')).

preserve.
SET Olang=English OVars names ONumbers values TVars names TNumbers values.
omsend.

/* nWithin is the number of within group variables */
!if (!upcase(!within)="NOWITHIN") !then
!let !nWithin=!null
!Else
!do !w !in (!within)
!let !nwithin=!concat(!nWithin," ")
!doend !ifend.
!let !nwithin=!length(!nwithin).

/* z is nWithin + 2 */
!if (!nWithin>0) !then
!let !z=!length(!concat(!blanks(!nWithin),!blanks(2))).
!else
!let !z=3.
!ifend.
*title !concat('number of Within group variables is ',!nWithin,', plus 2 (z) = ',!z).

/* ncat is the number of categories
!let !nc=!null.
!do !c !in (!categ) !let !nc=!concat(!nc," ") !doend.
!let !ncat=!length(!nc).
*title !concat("Number of categories=",!ncat).

get file !quote(!infile) /keep !dep !if (!nwithin>0) !then !within !ifend !compare !wgt !concat(!rwgt,1) to !concat(!rwgt,!nrep).
!if (!nwithin=0) !then compute !within=1 !ifend.
compute MISS = (missing(!compare)).
!do !d !in (!dep).
if  (missing(!d)) MISS=1.
!doend.
!do !w !in (!within).
if (missing(!w)) MISS=1. 
!doend.
!let !n=!null.
!do !c !in(!categ)
!let !n=!concat(!n,'| ',!compare,'=',!c,!blanks(1))
!doend.
!let !n=!concat('(',!substr(!n,2),')').
*title !n.
compute VAL=!n.
if (VAL=0) MISS=1.
compute VALIDN=1-MISS.
formats VALIDN (f1.0).
save outfile='c:\temp\data.sav' /drop=MISS.

*------------ Next table with counts does not work without licence for CLABELS ---------.
*!let !q=!null.
*!if (!nwithin>0) !then
*!do !w !in(!within).
*!let !q=!concat(!q,!w,' [C] > ').
*!doend !ifend.
**title !q.
*
*VARIABLE LEVEL !within !compare (NOMINAL).
*string INCLUDED (a3).
*if (validn=0) INCLUDED='No'.
*if (validn=1) INCLUDED='Yes'.
*weight by !wgt.
*CTABLES
*  /VLABELS VARIABLES=!if (!nwithin>0) !then !within !ifend !compare INCLUDED DISPLAY=DEFAULT
*  /TABLE !q !compare [C][UCOUNT F40.0, COUNT F40.0] BY INCLUDED [C]
*  /CATEGORIES VARIABLES=!if (!nwithin>0) !then !within !ifend !compare INCLUDED ORDER=A KEY=VALUE EMPTY=EXCLUDE MISSING=INCLUDE
*  /TITLES TITLE='Columns "INCLUDED=Yes" give the weighted and unweighted number of included students'.
*---------------------------------------------------------------------------.
oms /select tables headings texts/destination viewer=no /tag="NoViewer".


*** COMPUTE ESTIMATES ***.

oms /select tables /if subtypes='Report' /destination format=sav outfile=!quote(!concat('c:\temp\',!wgt,'_1.sav')) /tag="fullweight".
select if (VALIDN=1).
weight by !wgt.
means !dep !if (!nWithin>0) !then !do !w !in (!within) !concat("by ",!w) !doend !else by !within !ifend by !compare /cells=MEAN VARIANCE.
omsend tag="fullweight".

Get file=!quote(!concat('c:\temp\',!wgt,'_1.sav')) /keep= Var1 to !concat('var',!z) !dep .
rename vars (Var1 to !concat('var',!z) = !within !compare TEMP).
compute y=number(!compare,f3.0).
select if (not missing(y)).
compute !compare=concat('cat',!compare).
string S (a3).
if (mod($casenum,2)=1) S='MN'.
if (mod($casenum,2)=0) S='VAR'.
sort cases by !within !compare S.
delete var y TEMP.
casestovars /id=!within /index=!compare S/groupby=index.

/* This part computes the difference between each pair of categories */. 
*--------------------------------------------------------------------------------------------------------.
!let !vars1=!null.
!do !d !in (!dep).
!let !a=!categ.
!do !x !in (!categ)
!do !c !in (!tail(!a)) 
!if (!tail(!a) > !null) !then
compute !concat(!d,".",!x,"_",!c)=(!concat(!d,".cat",!x,'.MN') - !concat(!d,".cat",!c,'.MN'))/
  sqrt((!concat(!d,".cat",!x,'.VAR') + !concat(!d,".cat",!c,'.VAR'))/2).
!let !vars1=!concat(!vars1,!concat(!d,".",!x,"_",!c)," ").
*title !d # !x # !c.
!ifend !doend.
!let !a=!tail(!a).
!doend.
!doend.
*title 'End of 3 loops'.
exe.
*--------------------------------------------------------------------------------------------------------.
!let !vars2=!null.
!do !d !in (!dep) !do !c !in (!categ)
!let !vars2=!concat(!vars2,!d,'.cat',!c,'.MN ',!d,'.cat',!c,'.VAR ').
!doend !doend
delete var !vars2.

VARSTOCASES
  /MAKE STAT FROM !vars1
  /INDEX=TEMP(STAT) 
  /KEEP=!within 
  /NULL=KEEP.
!let !q=!length(!concat(!head(!dep),!blanks(2))).
string PV (!concat('a',!length(!head(!dep)))).
compute PV=substr(TEMP,1,!length(!head(!dep))).
string !upcase(!compare) (a10).
compute !compare=substr(TEMP,!q).
exe.

!let !f=!null.
!do !d !in (!dep).
!let !f=!concat(!f,!blanks(1)).
!let !g=!length(!f).
recode PV (!quote(!d)=!quote(!concat('PV',!g))).
!doend.
sort cases by !within !compare PV.
delete var TEMP.
casestovars /id= !within !compare /index=PV.
rename vars (pv1 to pv5 = !dep).
save outile=!quote(!concat('c:\temp\',!wgt,'.sav')).


**** REPLICATES ***.

dataset close all.
get file='c:\temp\data.sav'.
select if (VALIDN=1).
oms /select tables /if subtypes='Report' /destination format=sav numbered='REP' outfile=!quote(!concat('c:\temp\',!rwgt,'_1.sav')) /tag="r".
!do !i=1 !to !nrep.
weight by !concat(!rwgt,!i).
means !dep !if (!nWithin>0) !then !do !w !in (!within) !concat("by ",!w) !doend !else by !within !ifend by !compare /cells=MEAN VARIANCE.
!doend.
omsend tag="r".

get file=!quote(!concat('c:\temp\',!rwgt,'_1.sav')) /keep= REP Var1 to !concat('var',!z) !dep .
rename vars (Var1 to !concat('var',!z) = !within !compare TEMP).
compute y=number(!compare,f3.0).
select if (not missing(y)).
compute !compare=concat('cat',!compare).
string S (a3).
if (mod($casenum,2)=1) S='MN'.
if (mod($casenum,2)=0) S='VAR'.
sort cases by !within REP !compare S.
delete var y TEMP.
casestovars /id=!within REP /index=!compare S/groupby=index.

/* This part computes the difference between each pair of categories */. 
*--------------------------------------------------------------------------------------------------------.
!let !vars1=!null.
!do !d !in (!dep).
!let !vars=!null.
!let !a=!categ.
!do !x !in (!categ)
!do !c !in (!tail(!a)) 
!if (!tail(!a) > !null) !then
compute !concat(!d,".",!x,"_",!c)=(!concat(!d,".cat",!x,'.MN') - !concat(!d,".cat",!c,'.MN'))/
  sqrt((!concat(!d,".cat",!x,'.VAR') + !concat(!d,".cat",!c,'.VAR'))/2).
!let !vars1=!concat(!vars1,!concat(!d,".",!x,"_",!c)," ").
*title !d # !x # !c.
!ifend !doend.
!let !a=!tail(!a).
!doend.
!doend.
*title 'End of 3 loops'.
exe.
*--------------------------------------------------------------------------------------------------------.
!let !vars2=!null.
!do !d !in (!dep) !do !c !in (!categ)
!let !vars2=!concat(!vars2,!d,'.cat',!c,'.MN ',!d,'.cat',!c,'.VAR ').
!doend !doend
delete var !vars2.

VARSTOCASES
  /MAKE STATR FROM !vars1
  /INDEX=TEMP(STATR) 
  /KEEP=!within REP
  /NULL=KEEP.
!let !q=!length(!concat(!head(!dep),!blanks(2))).
string PV (!concat('a',!length(!head(!dep)))).
compute PV=substr(TEMP,1,!length(!head(!dep))).
string !upcase(!compare) (a10).
compute !compare=substr(TEMP,!q).
exe.

!let !f=!null.
!do !d !in (!dep).
!let !f=!concat(!f,!blanks(1)).
!let !g=!length(!f).
recode PV (!quote(!d)=!quote(!concat('PV',!g))).
!doend.
sort cases by !within !compare REP PV.
delete var TEMP.
casestovars /id= !within !compare REP /index=PV.
rename vars (pv1 to pv5 = !do !d !in(!dep) !concat('r_',!d) !doend).


*** COMBINE RESULTS ***.

sort cases by !within !compare.
match files table=!quote(!concat('c:\temp\',!wgt,'.sav')) /file=* /by !within !compare.
exec.


*** COMPUTE SAMPLING VARIANCE (U) ***.

!do !d !in(!dep) 
COMPUTE !concat('var_',!d) = !cons*((!concat('r_',!d) - !d)**2).   
!doend.

AGGREGATE OUTFILE =*
   /PRESORTED /BREAK= !within !compare 
   /!do !d !in(!dep) !d !doend=MEAN(!do !d !in(!dep) !d !doend) 
   /!do !d !in(!dep) !concat('var_',!d) !doend=SUM(!do !d !in(!dep) !concat('var_',!d) !doend) .

!let !k=!null.
!do !d !in (!dep)
!let !k=!concat(!k,",var_",!d).
!doend.
compute pv_var=mean(!substr(!k,2)).
exe.



*** CALCULATING MEASUREMENT VARIANCE (Bm) ***.

!let !k=!null.
!do !d !in (!dep)
!let !k=!concat(!k,",",!d).
!doend.
compute STAT=mean(!substr(!k,2)).

do repeat a=!dep/b=pvar1 to pvar5.
compute b=(a-STAT)**2.
end repeat.

compute pvmerr=.25*(sum(pvar1 to pvar5)).



*** COMPUTE SE [V = U + (1+1/M)Bm] ***.

compute SE=sqrt(pv_var+1.2*pvmerr).
exe.
FORMATS STAT (F8.3) / SE (F10.6).
omsend tag="NoViewer".



*** OUTPUT TABLE ***.

OMS /SELECT TABLES headings
   /IF COMMANDS = ["Summarize"] SUBTYPES = ["Case Processing Summary"]
   /DESTINATION   VIEWER = NO /TAG = "SumNoCaseProc".

SUMMARIZE
  /TABLES=!if (!nWithin>0) !then !within !ifend !compare stat se
  /FORMAT=LIST NOCASENUM NOTOTAL 
  !if (!nwithin=0) !then /TITLE=!quote(!concat('SE on effect size in ',!upcase(!head(!dep)),' to PV5 between ',!upcase(!compare),' groups'))
  !else /TITLE=!quote(!concat('SE on effect size in ',!upcase(!head(!dep)),' to PV5 between ',!upcase(!compare),' groups by ',!upcase(!within))) !ifend
  /CELLS=NONE.
OMSEND TAG=["SumNoCaseProc"].

delete variables !dep !do !d !in (!dep) !concat("var_",!d) !doend !if (!nWithin=0) !then !within !ifend pv_var pvar1 to pvar5 pvmerr .
save outfile="c:\temp\results.sav".
restore.

!enddefine.