set mprint=no.

*** Created by Eveline Gebhardt, Australian Council for Educational Research ***.

define pctile  (pv    =!charend('/')
               /ptiles=!charend('/')
               /infile=!charend('/')
               /grp   =!default(NOGRP) !charend('/')
               /wgt   =!default(W_FSTUWT) !charend('/')
               /rwgt  =!default(W_FSTR) !charend('/')
               /nrep  =!default(80) !charend('/')
               /cons  =!default(0.05) !charend('/')).
preserve.
SET Olang=English OVars names ONumbers values TVars names TNumbers values mxwarns=0.
omsend.
oms /select headings tables texts/destination viewer=no /tag="crap".

!let !a=!null.
!do !b !in (!ptiles)
!let !a=!concat(!a,!blanks(1)).
!doend.
!let !nptile=!length(!a).

!let !a=!null.
!do !b !in (!grp)
!let !a=!concat(!a,!blanks(1)).
!doend.
!let !ngrp=!length(!a).
!if (!upcase(!grp)=NOGRP) !then !let !ngrp=0 !ifend.
!let !plus1=!length(!concat(!a,!blanks(1))).
!let !plus2=!length(!concat(!a,!blanks(2))).

!let !k=!null.
!do !d !in (!pv).
!let !k=!concat(!k,",",!d).
!doend.
!let !k=!substr(!k,2).

!let !l=!null.
!do !d !in (!pv).
!let !l=!concat(!l,",var_",!d).
!doend.
!let !l=!substr(!l,2).

dataset close all.
get file=!quote(!infile) /keep=!if (!ngrp=0) !then !else !grp !ifend !pv !wgt !concat(!rwgt,'1') to !concat(!rwgt,!nrep).
!if (!ngrp=0) !then compute !grp=1 !ifend.
sort cases by !grp.
save outfile='c:\temp\sort.sav'.


*** FULL WEIGHT ***.

dataset close all.
get file='c:\temp\sort.sav'.
oms /select tables /if subtype='Statistics' /destination format=sav outfile='c:\temp\fwgt.sav' /tag='fwgt'.
split file layered by !grp.
weight by !wgt.
fre !pv /format=notable /percentiles !ptiles.
split file off.
omsend tag='fwgt'.
dataset close all.
get file='c:\temp\fwgt.sav' /drop=Command_ Subtype_ Label_ !concat('var',!plus1).
compute temp=number(!concat('var',!plus2),f7.0).
select if not missing (temp).
RENAME VAR (!if (!ngrp=0) !then var1=!grp !else var1 to !concat('var',!ngrp)=!grp) !ifend.
save outfile='c:\temp\fwgt2.sav'.


**** REPLICATE WEIGHTS ***.

dataset close all.
get file='c:\temp\sort.sav'.
oms /select tables /if subtype='Statistics' /destination format=sav outfile='c:\temp\rwgt.sav' /tag='rwgt'.
split file layered by !grp.
!do !i = 1 !to !nrep.
weight by !concat(!rwgt,!i).
fre !pv /format=notable /percentiles !ptiles.
!doend.
split file off.
omsend tag='rwgt'.
dataset close all.
get file='c:\temp\rwgt.sav' /drop=Command_ Subtype_ Label_ !concat('var',!plus1).
compute temp=number(!concat('var',!plus2),f7.0).
select if not missing (temp).
rename vars (!pv=!do !d !in (!pv) !concat("R_",!d) !doend).
RENAME VAR (!if (!ngrp=0) !then var1=!grp !else var1 to !concat('var',!ngrp)=!grp !ifend).
sort case by !grp temp.
save outfile='c:\temp\wgt2.sav'.

match files table='c:\temp\fwgt2.sav' /file='c:\temp\wgt2.sav' /by !grp temp.


*** CALCULATION SAMPLING VARIANCE ***.

!do !a !in (!pv).
COMPUTE !concat('var_',!a) = !cons * ((!concat('r_',!a) - !a)**2).   
!doend.

AGGREGATE OUTFILE =*
   /PRESORTED /BREAK= !grp temp
   /!do !d !in (!pv) !d !doend=MEAN(!do !d !in(!pv) !d !doend) 
   /!do !d !in (!pv) !concat('var_',!d) !doend=SUM(!do !d !in(!pv) !concat('var_',!d) !doend) .

compute pv_var=mean(!l).
exe.


*** CALCULATING MEASUREMENT VARIANCE (Bm) ***.

compute STAT=mean(!k).

do repeat a=!pv/b=pvar1 to pvar5.
compute b=(a-STAT)**2.
end repeat.

compute pvmerr=.25*(sum(pvar1 to pvar5)).
exe.


*** COMPUTE SE [V = U + (1+1/M)Bm] ***.

compute SE=sqrt(pv_var+1.2*pvmerr).
exe.
FORMATS STAT (F8.3) / SE (F10.6).
delete variables !pv !do !d !in (!pv) !concat("var_",!d) !doend pv_var pvar1 pvar2 pvar3 pvar4 pvar5 pvmerr.
formats temp (f3.0).
casestovars /id=!grp /index=temp /groupsby index.
omsend tag="crap".


*** OUTPUT TABLE ***.

!if (!nptile=1) !then rename var (STAT SE=!concat('STAT.',!ptiles) !concat('SE.',!ptiles)) !ifend.
!let !var=!null.
!do !p !in (!ptiles).
!let !var=!concat(!var,' STAT.',!p,' SE.',!p).
!doend.

Oms /select tables /if subtypes='Case Processing Summary' /destination viewer=no /tag="case".
SUMMARIZE
  /TABLES= !if (!ngrp=0) !then !else !grp !ifend !var
  /FORMAT=LIST NOCASENUM
  !if (!ngrp=0) !then /TITLE=!quote(!concat("Percentiles ",!head(!dep)," to ",!substr(!dep,!Index(!pv,PV5)))) 
  !else /TITLE=!quote(!concat("Percentiles ",!head(!pv)," to ",!substr(!pv,!Index(!pv,PV5))," by ",!grp)) !ifend
  /CELLS=NONE.
omsend tag="case".

!if (!ngrp=0) !then delete var !grp !ifend.

save outfile="c:\temp\results.sav".

RESTORE.
!enddefine.

