set mprint=no.

*** Created by Eveline Gebhardt & Alexander Daraganov, Australian Council for Educational Research ***.

define REGnoPV (nrep = !default(80) !charend('/')/
                ind = !charend('/') /
                dep = !charend('/') /
                grp = !default(NOGRP) !charend('/') /
                wgt = !default(W_FSTUWT) !charend('/') /
                rwgt = !default(W_FSTR) !charend('/') /
                cons = !default(0.05) !charend('/')/
                PSU = !default(SCHOOLID) !charend('/')/
                limit_criteria = !default(0) !charend('/')/
                infile = !charend('/')).

preserve.
SET Olang=English OVars names ONumbers values TVars names TNumbers values.
omsend.
oms /select headings tables texts/destination viewer=no /tag="crap".

/* nGrp is the number of group variables */
!if (!upcase(!grp)=NOGRP) !then
!let !nGrp=!null
!Else
!do !g !in (!grp)
!let !nGrp=!concat(!nGrp," ")
!doend !ifend.
!let !nGrp=!length(!nGrp).
title !quote(!concat('Number of group variables = ',!ngrp)).

get file !infile /keep=!if (!ngrp>0) !then !grp !ifend !PSU !wgt !concat(!rwgt,1) to !concat(!rwgt,!nrep) !dep !ind.
!if (!ngrp=0) !then compute !grp=1 !ifend.
missing values !grp ().
sort cases by !grp.       
compute MISS = (missing(!dep)).
!do !i !in (!ind).
if (missing(!i)) MISS=1.
!doend.
compute VALIDN=1-MISS.
formats VALIDN (f1.0).
save outfile='c:\temp\sort.sav' /drop=MISS.


*** COMPUTE ESTIMATES ***.
     
get file 'c:\temp\sort.sav'.
split file by !grp.            
weight by !wgt.

oms /select tables /if subtype='Model Summary' /destination format=sav outfile='c:\temp\R2.sav' /TAG="R2".
REGRESSION
  /missing=listwise
  /DEPENDENT !dep
  /METHOD=ENTER !ind
  /OUTFILE=COVB('C:\temp\coef.sav') .
omsend tag="R2".

get file='C:\temp\coef.sav'.
select if (rowtype_='EST').
rename var(CONST_=@INTERCEPT).
VARSTOCASES  /MAKE STAT FROM @INTERCEPT !ind
 /INDEX = ind(stat) /KEEP = !grp /NULL = KEEP.
Recode ind ('@INTERCEPT'=' INTERCEPT').
save outfile='c:\temp\all.sav'.

get file='c:\temp\R2.sav' /keep=!if (!ngrp=0) !then var1 !else !do !n=1 !to !nGrp !concat('var',!n," ") !doend !ifend RSquare.
rename var (!if (!ngrp=0) !then var1 !else !do !n=1 !to !nGrp !concat('var',!n," ") !doend !ifend RSquare=!grp STAT).
formats stat (f8.2).
string IND (a10).
compute IND="  R-SQUARE".
add files file='c:\temp\all.sav' /file=*.
sort cases by !grp ind.
save outfile='c:\temp\all2.sav'.


* REPLICATES.
oms /select tables /if subtype='Model Summary' /destination format=sav outfile='c:\temp\R2R.sav' /TAG="R2R".
!do !i=1 !to !nrep.
dataset close all.
get file='c:\temp\sort.sav'.

*sort cases by !grp.            
split file by !grp.            
                              
weight by !concat(!rwgt,!i).
REGRESSION
  /missing=listwise
  /DEPENDENT !dep
  /METHOD=ENTER !ind
  /OUTFILE=COVB('C:\temp\coef.sav') .

get file='C:\temp\coef.sav'.
select if (rowtype_='EST').
rename var(CONST_=@INTERCEPT).
VARSTOCASES  /MAKE statR FROM @INTERCEPT !ind
 /INDEX = ind(statR) /KEEP = !grp /NULL = KEEP.
Recode ind ('@INTERCEPT'=' INTERCEPT').

save outfile=!quote(!concat('C:\temp\',!dep,!i,'.sav')).
!doend.
omsend tag="R2R".


*** COMBINE RESULTS ***.

dataset close ALL.
get file =!quote(!concat('C:\temp\',!dep,'1.sav')).

!let !ccc=!NULL.
!Do !e = 2 !to !nrep.
add files file=* /file=!quote(!concat('c:\temp\',!dep,!e,'.sav')).
exe.
!if ((!e=50) !or (!e=100) !or (!e=150) !or (!e=200) !or (!e=250) !or (!e=300) !or (!e=350) !or (!e=400) !or (!e=450)) !then
!let !ccc=(!Concat(!ccc,!Blank(1))).
save outfile=!quote(!concat('c:\temp\',!dep,'_temp_',!length(!ccc),'.sav')).
!let !f=!length(!concat(!blanks(!e),!blanks(1))).
get file =!quote(!concat('C:\temp\',!dep,!f,'.sav')).
select if $casenum=0.
!ifend
!if (!e=!nrep) !then
!let !ccc=(!Concat(!ccc,!Blank(1))).
save outfile=!quote(!concat('c:\temp\',!dep,'_temp_',!length(!ccc),'.sav')).
!if (!nrep>50) !then
add files !do !cccc=1 !to !length(!ccc)/file=!quote(!concat('c:\temp\',!dep,'_temp_',!cccc,'.sav')) !doend.
!ifend
!ifend
!Doend
save outfile='c:\temp\rwgt.sav'.

get file='c:\temp\R2R.sav' /keep=!if (!ngrp=0) !then var1 !else !do !n=1 !to !nGrp !concat('var',!n," ") !doend !ifend RSquare.
rename var (!if (!ngrp=0) !then var1 !else !do !n=1 !to !nGrp !concat('var',!n," ") !doend !ifend RSquare=!grp STATR).
formats statr (f8.2).
string IND (a10).
compute IND="  R-SQUARE".
add files file='c:\temp\rwgt.sav' /file=*.
sort cases by !grp ind.
save outfile='c:\temp\rwgt2.sav'.

sort cases by !grp ind.
match files file=* /table='c:\temp\all2.sav' /by !grp ind.
exec.


*** COMPUTE SAMPLING VARIANCE (U) ***.

compute var=(statr-stat)**2.

aggregate outfile=* /break=!grp IND /stat= mean(stat) /var=sum(var).
compute var=!cons*var.


*** COMPUTE SE ***.

compute se=sqrt(var).
formats stat (f8.3)/ SE (f10.6).
save outfile='c:\temp\stats.sav'.


*** COMPUTE N ***.

!let !z=!substr(!limit_criteria,!length(!limit_criteria),1).
get file='c:\temp\sort.sav'.
weight by !wgt.
aggregate /break=!if (!z=1) !then !head(!grp) !else !grp !ifend /N_total=N.
filter by VALIDN.
aggregate outfile='c:\temp\y.sav' /break=!grp /NU_cases=NU /N_cases=N.
weight off.
aggregate outfile=* /break=!grp !PSU /N=N /N_total=mean(N_total).
aggregate outfile=* /break=!grp /NU_PSU=N /N_total=mean(N_total).
match files /file='c:\temp\y.sav' /file=* /by !grp.
match files file='c:\temp\stats.sav' /TABLE=* /by !grp.
formats N_cases (f10.0).
filter off.


*** FLAGGING FOR OECD ***.

!if (!limit_criteria=0) !then !else
!let !a=!null.
!do !b !in (!limit_criteria).
!let !a=!concat(!a," ").
!if (!length(!a)=1) !then.
compute FLAG_std=(NU_cases<!b).
formats FLAG_std (f1.0).
!ifend.
!if (!length(!a)=2) !then.
compute FLAG_sch=(NU_PSU<!b).
formats FLAG_sch (f1.0).
!ifend.
!if (!length(!a)=3) !then
compute FLAG_pct=((100*N_cases/N_total)<!b).
formats FLAG_pct (f1.0).
!ifend.
!doend.
!ifend.
compute IND=Ltrim(IND).
exe.
omsend tag="crap".

*** OUTPUT TABLES ***.

oms /select tables /if subtypes='Case Processing Summary' /destination viewer=no /tag="case".
SUMMARIZE
  /TABLES= !if (!nGrp>0) !then !grp !ifend IND stat SE !if (!limit_criteria=0) !then !else FLAG_std FLAG_sch FLAG_pct !ifend
  /FORMAT=LIST NOCASENUM
  !if (!ngrp=0) !then /TITLE=!quote(!concat("Regression of ",!dep," on ",!ind)) 
  !else /TITLE=!quote(!concat("Regression of ",!dep," on ",!ind," by ",!grp)) !ifend
  /CELLS=NONE.

delete variables !if (!limit_criteria=0) !then N_total !ifend !if (!ngrp=0) !then !grp !ifend  var.
omsend tag="case".
restore.

!if (!limit_criteria=0) !then.
save outfile="c:\temp\results.sav" drop= n_cases nu_cases nu_psu.
get file="c:\temp\results.sav".
!else.
save outfile="c:\temp\results.sav" drop=n_cases nu_cases nu_psu n_total.
get file="c:\temp\results.sav".
!ifend.

!enddefine.


