set mprint=no.

*** Created by Eveline Gebhardt & Alexander Daraganov, Australian Council for Educational Research ***.

define REG_PV (infile = !charend('/')/
               ind = !charend('/')/
               dep = !charend('/')/
               grp = !default(NOGRP) !charend('/')/
               nrep = !default(80) !charend('/')/
               wgt = !default(W_FSTUWT) !charend('/')/
               rwgt = !default(W_FSTR) !charend('/')/
               cons = !default(0.05) !charend('/')/
               PSU = !default(SCHOOLID) !charend('/')/
               limit_criteria = !default(0) !charend('/')).

preserve.
SET Olang=English OVars names ONumbers values TVars names TNumbers values.
omsend.
oms /select headings tables texts/destination viewer=no /tag="crap".

/* nGrp is the number of group variables */
!let !nGrp=!null
!if (!upcase(!grp)=NOGRP) !then !else
!do !g !in (!grp)
!let !nGrp=!concat(!nGrp," ")
!doend !ifend.
!let !nGrp=!length(!nGrp).
title !quote(!concat('Number of group variables = ',!ngrp)).

get file !infile /keep=!if (!ngrp>0) !then !grp !ifend !PSU !wgt !concat(!rwgt,1) to !concat(!rwgt,!nrep) !dep !ind .
!if (!ngrp=0) !then compute !grp=1 !ifend.
missing values !grp ().
sort cases by !grp. 
compute MISS = 0. 
!do !d !in (!dep).     
if (missing(!d)) MISS=1.
!doend.
!do !i !in (!ind).
if (missing(!i)) MISS=1.
!doend.
compute VALIDN=1-MISS.
formats VALIDN (f1.0).
save outfile='c:\temp\sort.sav' /drop=MISS.


*** COMPUTE REGRESSION COEFFICIENTS ***.

get file 'c:\temp\sort.sav'.
split file by !grp.            
weight by !wgt.

oms /select tables /if subtype='Model Summary' /destination format=sav numbered=PV outfile='c:\temp\R2.sav' /TAG="R2".
REGRESSION
  /missing=listwise
  /DEPENDENT !dep
  /METHOD=ENTER !ind
  /OUTFILE=COVB(!quote(!concat('C:\temp\coef.sav'))) .
omsend tag="R2".

get file='C:\temp\coef.sav'.
select if (rowtype_='EST').
rename var(CONST_=@INTERCEPT).
sort cases by !grp depvar_.
VARSTOCASES 
   /MAKE s FROM @INTERCEPT !ind
   /INDEX = IND(s) /KEEP =  DEPVAR_ !grp /NULL = KEEP.
sort cases by !grp IND depvar_.
casestovars /id=!grp IND /index=depvar_.
rename vars (!dep=stat.1 to stat.5).
Recode ind ('@INTERCEPT'=' INTERCEPT').
save outfile='c:\temp\all.sav'.

get file='c:\temp\R2.sav'
 /keep=PV !if (!ngrp=0) !then var1 !else !do !n=1 !to !nGrp !concat('var',!n," ") !doend !ifend RSquare.
rename var (!if (!ngrp=0) !then var1 !else !do !n=1 !to !nGrp !concat('var',!n," ") !doend !ifend RSquare=!grp STAT).
formats stat (f8.2).
sort cases by !grp.
casestovars /id=!grp /index=PV.
string IND (a10).
compute IND="  R-SQUARE".
add files file='c:\temp\all.sav' /file=*.
sort cases by !grp ind.
save outfile='c:\temp\all2.sav'.


* REPLICATES.

oms /select tables /if subtype='Model Summary' /destination format=sav numbered=PVREP outfile='c:\temp\R2R.sav' /TAG="R2R".
!do !i=1 !to !nrep.
dataset close all.
get file 'c:\temp\sort.sav'.
split file by !grp.            
                              
weight by !CONCAT(!rwgt,!i).
REGRESSION
  /missing=listwise
  /DEPENDENT !dep
  /METHOD=ENTER !ind
  /OUTFILE=COVB(!quote(!concat('C:\temp\rep',!i,'.sav'))) .

get file=!quote(!concat('C:\temp\rep',!i,'.sav')).
select if (rowtype_='EST').
rename var(CONST_=@INTERCEPT).
sort cases by !grp depvar_.
VARSTOCASES 
   /MAKE s FROM @INTERCEPT !ind
   /INDEX = IND(s)
   /KEEP =  DEPVAR_ !grp
   /NULL = KEEP.
sort cases by !grp IND depvar_.
casestovars /id=!grp IND /index=depvar_.
rename vars (!dep=statR.1 to statR.5).
Recode ind ('@INTERCEPT'=' INTERCEPT').
save outfile=!quote(!concat('C:\temp\',!rwgt,!i,'.sav')).
!doend.
omsend tag="R2R".

dataset close ALL.


* Combine replicate files.

!let !nper=50.
!if ( !length(!substr(!blanks(!nrep),!length(!concat(!blanks(!nper ),!blanks(1)))))=0 ) !then
	add files file= !quote(!concat('C:\temp\',!rwgt,'1.sav')) !do !e=2 !to !nrep /file=!quote(!concat('C:\temp\',!rwgt,!e,'.sav')) !doend .
!else
	!let !modls=!nrep.
	!let !intgr=0.
	!Do !iter=!nrep !to !nper !by -!nper
		add files file= !quote(!concat('C:\temp\',!rwgt,!iter,'.sav')) !do !e=!length(!substr(!blanks(!iter),2)) !to !length(!substr(!blanks(!iter),!length(!blanks(!nper)))) !by -1 /file=!quote(!concat('C:\temp\',!rwgt,!e,'.sav')) !doend .
		!let !modls=!length(!substr(!blanks(!modls),!length(!concat(!blanks(1),!blanks(!nper))))).
		!let !intgr=!length(!Concat(!blanks(!intgr), !blanks(1))).
		save outfile=!quote(!concat('c:\temp\',!rwgt,'_temp_',!intgr,'.sav')).
	!doend
	!if (!modls>0) !then
		!if (!modls>1) !then
			add files file= !quote(!concat('C:\temp\',!rwgt,'1.sav')) !do !e=2 !to !modls /file=!quote(!concat('C:\temp\',!rwgt,!e,'.sav')) !doend .
			!let !intgr=!length(!Concat(!blanks(!intgr), !blanks(1))).
			save outfile=!quote(!concat('c:\temp\',!rwgt,'_temp_',!intgr,'.sav')).
		!else
			get file =!quote(!concat('C:\temp\',!rwgt,'1.sav')).
			!let !intgr=!length(!Concat(!blanks(!intgr), !blanks(1))).
			save outfile=!quote(!concat('c:\temp\',!rwgt,'_temp_',!intgr,'.sav')).
		!ifend
	!ifend
	!if ( !length(!substr(!blanks(!intgr),!length(!concat(!blanks(!nper ), !blanks(1)))))=0 ) !then
		add files file= !quote(!concat('C:\temp\',!rwgt,'_temp_1.sav')) !do !e=2 !to !intgr /file=!quote(!concat('C:\temp\',!rwgt,'_temp_',!e,'.sav')) !doend .
	!else
		!let !modls=!intgr.
		!let !intgr1=0.
		!do !iter=!intgr !to !nper !by -!nper
			add files file= !quote(!concat('C:\temp\',!rwgt,'_temp_',!iter,'.sav')) !do !e=!length(!substr(!blanks(!iter),2)) !to !length(!substr(!blanks(!iter),!length(!blanks(!nper)))) !ny -1 /file=!quote(!concat('C:\temp\',!rwgt,'_temp_',!e,'.sav')) !doend.
			!let !modls=!length(!substr(!blanks(!modls),!length(!concat(!blanks(1),!blanks(!nper))))).
			!let !intgr1=!length(!Concat(!blanks(!intgr1), !blanks(1))).
			save outfile=!quote(!concat('c:\temp\',!rwgt,'_tempover_',!intgr1,'.sav')).
		!doend
		!if (!modls>0) !then
			!if (!modls>1) !then
				add files file= !quote(!concat('C:\temp\',!rwgt,'_temp_1.sav')) !do !e=2 !to !modls /file=!quote(!concat('C:\temp\',!rwgt,'_temp_',!e,'.sav')) !doend .
				!let !intgr1=!length(!Concat(!blanks(!intgr1), !blanks(1))).
				save outfile=!quote(!concat('c:\temp\',!rwgt,'_tempover_',!intgr1,'.sav')).
			!else
				get file= !quote(!concat('C:\temp\',!rwgt,'_temp_1.sav')).
				!let !intgr1=!length(!Concat(!blanks(!intgr1), !blanks(1))).
				save outfile=!quote(!concat('c:\temp\',!rwgt,'_tempover_',!intgr1,'.sav')).
			!ifend
		!ifend
		add files file= !quote(!concat('c:\temp\',!rwgt,'_tempover_1.sav')) !do !e=2 !to !intgr1 /file=!quote(!concat('C:\temp\',!rwgt,'_tempover_',!e,'.sav')) !doend .
	!ifend
!ifend
exe.
save outfile='c:\temp\rep1.sav'.

get file='c:\temp\R2R.sav'
 /keep=PVREP !if (!ngrp=0) !then var1 !else !do !n=1 !to !nGrp !concat('var',!n," ") !doend !ifend RSquare.
rename var (!if (!ngrp=0) !then var1 !else !do !n=1 !to !nGrp !concat('var',!n," ") !doend !ifend RSquare=!grp STATR).
formats statR (f8.2).
compute PV=mod(PVREP,5).
recode PV (0=5).
if (PV=5) REP=PVREP/PV.
sort cases by !grp PVREP (d).
!do !j=4 !to 1 !by -1.
if (PV=!j) REP=lag(REP).
exe.
!doend.
formats PV REP (F3.0).
sort cases by !grp REP PV.
delete var PVREP.
casestovars /id=!grp REP/index=PV.
string IND (a10).
compute IND="  R-SQUARE".
add files file=* /file='c:\temp\rep1.sav'.
save outfile='c:\temp\rep2.sav'.
delete var REP.

SORT CASES BY !grp IND.
match files file=*/table= 'c:\temp\all2.sav'/by !grp IND.
exec.


*** COMPUTE SAMPLING VARIANCE (U) ***.

do repeat a=statr.1 to statr.5/
	  b=stat.1 to stat.5/
	  c=var1 to var5.
compute c=(a-b)**2.
end repeat.

aggregate outfile=*/
	  break=!grp IND/ 
          stat.1 to stat.5= mean(stat.1 to stat.5)/
          var1 to var5 = sum(var1 to var5).

do repeat a=var1 to var5.
compute a=!cons*a.
end repeat.

compute pv_var=mean(var1 to var5).


*** CALCULATING MEASUREMENT VARIANCE (Bm) ***.

compute stat=mean(stat.1 to stat.5).
do repeat a=stat.1 to stat.5/b=pvar1 to pvar5.
compute b=(a-stat)**2.
end repeat.
compute pvmerr=.25*(sum(pvar1 to pvar5)).


*** COMPUTE SE [V = U + (1+1/M)Bm] ***.

compute SE=sqrt(pv_var+1.2*pvmerr).
formats stat (f8.3)/ SE (f10.6).
save outfile='c:\temp\stats.sav'.


*** COMPUTE N ***.

!let !z=!substr(!limit_criteria,!length(!limit_criteria),1).
get file='c:\temp\sort.sav'.
weight by !wgt.
aggregate /break=!if (!z=1) !then !head(!grp) !else !grp !ifend /N_total=N.
filter by VALIDN.
aggregate outfile='c:\temp\y.sav' /break=!grp /NU_cases=NU /N_cases=N.
weight off.
aggregate outfile=* /break=!grp !PSU /N=N /N_total=mean(N_total).
aggregate outfile=* /break=!grp /NU_PSU=N /N_total=mean(N_total).
match files /file='c:\temp\y.sav' /file=* /by !grp.
match files file='c:\temp\stats.sav' /TABLE=* /by !grp.
formats N_cases (f10.0).
filter off.


*** FLAGGING FOR OECD ***.

!if (!limit_criteria=0) !then !else
!let !a=!null.
!do !b !in (!limit_criteria).
!let !a=!concat(!a," ").
!if (!length(!a)=1) !then.
compute FLAG_std=(NU_cases<!b).
formats FLAG_std (f1.0).
!ifend.
!if (!length(!a)=2) !then.
compute FLAG_sch=(NU_PSU<!b).
formats FLAG_sch (f1.0).
!ifend.
!if (!length(!a)=3) !then
compute FLAG_pct=((100*N_cases/N_total)<!b).
formats FLAG_pct (f1.0).
!ifend.
!doend.
!ifend.
compute IND=Ltrim(IND).
exe.
omsend tag="crap".

*** OUTPUT TABLES ***.

Oms /select tables /if subtypes='Case Processing Summary' /destination viewer=no /tag="case".
SUMMARIZE
  /TABLES= !if (!nGrp>0) !then !grp !ifend IND stat SE !if (!limit_criteria=0) !then !else FLAG_std FLAG_sch FLAG_pct !ifend
  /FORMAT=LIST NOCASENUM
  !if (!ngrp=0) !then /TITLE=!quote(!concat("Regression of ",!head(!dep)," to ",!substr(!dep,!Index(!dep,PV5))," on ",!ind)) 
  !else /TITLE=!quote(!concat("Regression of ",!head(!dep)," to ",!substr(!dep,!Index(!dep,PV5))," on ",!ind," by ",!grp)) !ifend
  /CELLS=NONE.

omsend tag="case".

delete variables !if (!ngrp=0) !then !grp !ifend stat.1 stat.2 stat.3 stat.4 stat.5 
      var1 var2 var3 var4 var5 pv_var pvar1 pvar2 pvar3 pvar4 pvar5 pvmerr !if (!limit_criteria=0) !then N_total !ifend .

!if (!limit_criteria=0) !then.
save outfile="c:\temp\results.sav" drop= n_cases nu_cases nu_psu.
get file="c:\temp\results.sav".
!else.
save outfile="c:\temp\results.sav" drop=n_cases nu_cases nu_psu n_total.
get file="c:\temp\results.sav".
!ifend.

!enddefine.